// Copyright 2010 Google Inc. All Rights Reserved.

// BoundingBox.h: interface for the CBoundingBox3d class.

#ifndef SKETCHUP_SOURCE_COMMON_GEOMETRY_BOUNDINGBOX_H_
#define SKETCHUP_SOURCE_COMMON_GEOMETRY_BOUNDINGBOX_H_

#if _MSC_VER > 1000
#pragma once
#endif  // _MSC_VER > 1000

//===================================================================

#include "source/common/geometry/point2d.h"
#include "source/common/geometry/point3d.h"

//===================================================================

// Identifies side of b-box. also used by ray intersect.
namespace atlast {
namespace geometry {

enum EBoxSide {
  kSideNONE = 0,
  kSideMINX,
  kSideMAXX,
  kSideMINY,
  kSideMAXY,
  kSideMINZ,
  kSideMAXZ
};

//===================================================================

// A one dimension range of values
class CRange {
 public:
  CRange()
    : m_min(1.0e30),
      m_max(-1.0e30) {}
  CRange(double minVal, double maxVal)
    : m_min(minVal),
      m_max(maxVal) {}

  // Use default copy ctor, dtor, and assignment operator
  //CRange(const CRange& r)
  //	: m_min(r.m_min), m_max(r.m_max) {}
  //~CRange() {}

  //CRange& operator=(const CRange& r)
  //{
  //	m_min = r.m_min;
  //	m_max = r.m_max;
  //	return *this;
  //}

  // Set the values of the range
  void Set(double minVal, double maxVal) {
    m_min = minVal;
    m_max = maxVal;
  }

  // Clears the range.  A range is not valid after being cleared.
  void Clear() {
    m_min = 1.0e30;
    m_max = -1.0e30;
  }

  // Test to see if the range is empty (ie. nothing has been set.)
  bool IsEmpty() const {
    return m_max < m_min;
  }

  // Test to see if the range is valid (ie. not empty)
  bool IsValid() const {
    return !IsEmpty();
  }
  bool IsValidObj() const;

  // Get the extremes of the range
  double GetMin() const {
    return m_min;
  }
  double GetMax() const {
    return m_max;
  }

  // Get the size of the range
  double GetWidth() const {
    return m_max > m_min ? (m_max - m_min) : 0.0;
  }

  // Get the midpoint of the range
  double GetMid() const {
    return (m_min + m_max) / 2.0;
  }

  bool Contains(const double val) const {
    return val > (m_min - EqualTol) && val < (m_max + EqualTol);
  }

  bool Contains(const CRange &r) const {
    return r.m_min > (m_min - EqualTol) && r.m_max < (m_max + EqualTol);
  }

  // Test for containment in the interior of the range
  bool ContainsInInterior(double val) const;
  bool ContainsInInterior(const CRange &r) const;

  bool Intersects(const CRange &r) const;
  bool IntersectsInTolerance(const CRange& r, double tol) const;
  bool Overlaps(const CRange &r, CRange &overlap) const;

  // Add values to the range
  void operator+=(double val);
  void operator+=(const CRange &r);
  void Add(double val);

  // Create a new range that is the intersection of two ranges
  CRange Union(const CRange &r) const;
  CRange operator+(const CRange &r) const;
  CRange operator+(double val) const;

  // Return the intersection of two ranges
  CRange Intersect(const CRange &r) const;

  // Expand a range by a value at both ends.
  // A negative value will cause the range to shrink and could result
  // in an invalid range.
  void Expand(double val);

  friend bool operator<(double val, const CRange &r);
  friend bool operator>(double val, const CRange &r);
  bool operator<(const CRange &r) const {
    return m_max < r.m_min;
  }
  bool operator>(const CRange &r) const {
    return m_min > r.m_max;
  }

 private:
  double m_min;
  double m_max;
};

inline bool operator<(double val, const CRange &r) {
  return val < r.m_min;
}

inline bool operator>(double val, const CRange &r) {
  return val > r.m_max;
}

//===================================================================
// A 2D bounding box

class CBoundingBox2d {
 public:
  CBoundingBox2d()
      : m_xrange(),
        m_yrange() {}

  bool IsValidObj() const;

  // Use default copy ctor, dtor, and assignment operator
  //CBoundingBox2d(const CBoundingBox2d& box)
  //    : m_xrange(box.m_xrange), m_yrange(box.m_yrange) {}

  //~CBoundingBox2d() {}

  //CBoundingBox2d& operator=(const CBoundingBox2d& box)
  //{
  //    m_xrange = box.m_xrange;
  //    m_yrange = box.m_yrange;
  //    return *this;
  //}

  bool IsEmpty() const {
    return m_xrange.IsEmpty() || m_yrange.IsEmpty();
  }
  bool IsValid() const {
    return !IsEmpty();
  }

  // Clears the bounding box.
  // A bounding box is not valid after being cleared.
  void Clear();

  const CRange &GetXRange() const {
    return m_xrange;
  }
  const CRange &GetYRange() const {
    return m_yrange;
  }

  // Get the extents
  double GetXMin() const {
    return m_xrange.GetMin();
  }
  double GetXMax() const {
    return m_xrange.GetMax();
  }
  double GetYMin() const {
    return m_yrange.GetMin();
  }
  double GetYMax() const {
    return m_yrange.GetMax();
  }

  // Get min and max parameters by index
  double GetMin(int n) const {
    return n == 0 ? m_xrange.GetMin() : m_yrange.GetMin();
  }
  double GetMax(int n) const {
    return n == 0 ? m_xrange.GetMax() : m_yrange.GetMax();
  }

  // Get the dimensions
  double GetWidth() const {
    return m_xrange.GetWidth();
  }
  double GetHeight() const {
    return m_yrange.GetWidth();
  }
  double GetDiagonal() const;
  double GetDiagonalSquared() const;
  CPoint2d GetCenter() const;

  // Get a corner of the box.
  // 0 <= i <= 3
  CPoint2d GetCorner(int i) const;

  // Add points to the box
  void operator+=(const CPoint2d &pt);
  void operator+=(const CBoundingBox2d &box);
  void Add(const CPoint2d &pt);

  // Create the union of two boxes
  CBoundingBox2d Union(const CBoundingBox2d &box) const;
  CBoundingBox2d operator+(const CBoundingBox2d &box) const;
  CBoundingBox2d operator+(const CPoint2d &pt) const;

  // Return the intersection of two boxes
  CBoundingBox2d Intersect(const CBoundingBox2d &box) const;

  // Test whether this bounding box contains the given point
  bool Contains(const CPoint2d &pt) const;

  // Test if this bounding box contains the given one
  bool Contains(const CBoundingBox2d &box) const;

  // Test whether the given point is inside (ie. not on boundary of)
  // of the bounding box
  bool ContainsInInterior(const CPoint2d &pt) const;

  // Test whether the given bounding box inside this one.
  bool ContainsInInterior(const CBoundingBox2d &box) const;

  // Test whether the given bounding box intersects this one.
  bool Intersects(const CBoundingBox2d &box) const;

  // Test for overlap (not just touching) and return overlapping box
  bool Overlaps(const CBoundingBox2d &box, CBoundingBox2d &overlap) const;

  // Expand the bounding box
  void Expand(double val) {
    m_xrange.Expand(val);
    m_yrange.Expand(val);
  }
  void ExpandX(double val) {
    m_xrange.Expand(val);
  }
  void ExpandY(double val) {
    m_yrange.Expand(val);
  }

 private:  // data
  CRange m_xrange;
  CRange m_yrange;
};

//===================================================================

class CBoundingBox3d {
 public:
  CBoundingBox3d()
      : m_xrange(),
        m_yrange(),
        m_zrange() {}

  // Since we do not need any special stuff, we use the default
  // copy constructor, dtor, and assignment because they are faster.
  //CBoundingBox3d(const CBoundingBox3d& box);
  //~CBoundingBox3d() {}
  //CBoundingBox3d& operator=(const CBoundingBox3d& box);

  // Check to see if the range is valid.  If one of the ranges
  // gets set, they all will, so we really only need to check on.
  bool IsEmpty() const {
    return m_xrange.IsEmpty();
  }
  bool IsValid() const {
    return !IsEmpty();
  }
  bool IsValidObj() const;

  // Clears the bounding box.
  // A bounding box is not valid after being cleared.
  void Clear();

  const CRange &GetXRange() const {
    return m_xrange;
  }
  const CRange &GetYRange() const {
    return m_yrange;
  }
  const CRange &GetZRange() const {
    return m_zrange;
  }

  CPoint3d GetMinPoint() const;
  CPoint3d GetMaxPoint() const;
  CPoint3d GetMidPoint() const;
  double GetXMin() const {
    return m_xrange.GetMin();
  }
  double GetYMin() const {
    return m_yrange.GetMin();
  }
  double GetZMin() const {
    return m_zrange.GetMin();
  }
  double GetXMax() const {
    return m_xrange.GetMax();
  }
  double GetYMax() const {
    return m_yrange.GetMax();
  }
  double GetZMax() const {
    return m_zrange.GetMax();
  }

  // Get min and max parameters by index
  double GetMin(int n) const {
    return (n == 0 ? m_xrange.GetMin() :
            (n == 1 ? m_yrange.GetMin() :
             m_zrange.GetMin() ));
  }
  double GetMax(int n) const {
    return (n == 0 ? m_xrange.GetMax() :
            (n == 1 ? m_yrange.GetMax() :
             m_zrange.GetMax() ));
  }

  // Get the dimensions
  double GetWidth() const {
    return m_xrange.GetWidth();
  }
  double GetHeight() const {
    return m_yrange.GetWidth();
  }
  double GetDepth() const {
    return m_zrange.GetWidth();
  }

  double GetDiagonal() const;
  double GetDiagonalSquared() const;
  CPoint3d GetCenter() const;

  // Get a corner of the box.
  // 0 <= i <= 7
  CPoint3d GetCorner(int i) const;
  void SetCorner(int i, CPoint3d &pt) const;

  // Test whether this bounding box contains the given point
  bool Contains(const CPoint3d &pt) const;

  // Test if this bounding box contains the given one
  bool Contains(const CBoundingBox3d &box) const;

  // Test whether the given point is inside (ie. not on boundary of)
  // of the bounding box
  bool ContainsInInterior(const CPoint3d &pt) const;

  // Test whether the given bounding box inside this one.
  bool ContainsInInterior(const CBoundingBox3d &box) const;

  // Test to see if a line segment intersects the bounding box
  // NOTE: Points are passed by value because the algorithm may
  // modify them.
  bool Intersects(CPoint3d p1, CPoint3d p2) const;

  // Test whether the given bounding box intersects this one.
  bool Intersects(const CBoundingBox3d &box) const;
  bool IntersectsInTolerance(const CBoundingBox3d& box, double tol) const;

  // Test for overlap (not just touching) and return overlapping box
  bool Overlaps(const CBoundingBox3d &box, CBoundingBox3d &overlap) const;

  void operator+=(const CPoint3d &pt);
  void operator+=(const CBoundingBox3d &box);
  void Add(const CPoint3d &pt);

  // Create the union of two boxes
  CBoundingBox3d Union(const CBoundingBox3d &box) const;
  CBoundingBox3d operator+(const CBoundingBox3d &box) const;
  CBoundingBox3d operator+(const CPoint3d &pt) const;

  // Return the intersection of two boxes
  CBoundingBox3d Intersect(const CBoundingBox3d &box) const;

  // Return distance^2 to farthest point ... used for
  // calculating b-box expansion for perspective picking.
  double MaxDistanceSquaredTo(const CPoint3d &pt) const;

  // Expand b-box by given delta in all directions.
  // Negative delta will shrink.  Careful about shrinking
  // to much as the box will become inverted.
  void Expand(double delta);

  // Expand each b-box range by given ratio of maximum b-box range IFF the range is zero.
  // This is just trying to b-boxes with null sides.
  void ExpandRangeIfZero(double ratio);

  // Stretch b-box by vector at each corner
  void Stretch(const CVector3d &vec);

  void GetCenterNormal(EBoxSide side, CPoint3d &center, CUnitVector3d &normal);

 private:
  CRange m_xrange;
  CRange m_yrange;
  CRange m_zrange;
};

//===================================================================
// Stuff that was made inline as a result of profiling

inline void CRange::Add(double val) {
  if (val < m_min)
    m_min = val;
  if (val > m_max)
    m_max = val;
}

inline void CRange::operator+=(double val) {
  if (val < m_min)
    m_min = val;
  if (val > m_max)
    m_max = val;
}
}
}
//===================================================================
#endif  // SKETCHUP_SOURCE_COMMON_GEOMETRY_BOUNDINGBOX_H_
