// Copyright 2010 Google Inc. All Rights Reserved.

// Plane.h: interface for the CPlane class.
//
// We define a plane by the parameters a, b, c, d
// For any point on the plane, ax + by +cz + d = 0
// The coeficients are normalized so that a*a + b*b + c*c = 0
//
//////////////////////////////////////////////////////////////////////

#ifndef SKETCHUP_SOURCE_COMMON_GEOMETRY_PLANE_H_
#define SKETCHUP_SOURCE_COMMON_GEOMETRY_PLANE_H_

//===================================================================

#include "source/common/geometry/point2d.h"
#include "source/common/geometry/point3d.h"
#include "source/common/geometry/unitvector3d.h"
namespace atlast {
namespace geometry {

class CRay3d;
class CBoundingBox3d;
class CCircle3d;

#define PLANE_BBOX_ABOVE        1
#define PLANE_BBOX_BELOW        2
#define PLANE_BBOX_INTERSECTING 3

//===================================================================

class CPlane {
 public:
  enum ESide {
    kBelow    = -1,
    kCrossing =  0,
    kAbove    =  1
  };
  CPlane();
  CPlane(double a, double b, double c, double d);
  CPlane(const CPoint3d &pt, const CUnitVector3d &normal);
  CPlane(const CPoint3d &p1, const CPoint3d &p2, const CPoint3d &p3);

  // Use default copy ctor, dtor, and assignment operator
  //CPlane(const CPlane& plane);
  //~CPlane();
  //CPlane& operator=(const CPlane& plane);

  double DistanceTo(const CPoint3d &pt) const;
  double SignedDistanceTo(const CPoint3d &pt) const {
    return m_a * pt.X() + m_b * pt.Y() + m_c * pt.Z() + m_d;
  }

  // Trim the segment defined by pt1 and pt2 with this plane.  If both points
  // are above (signed distance > 0.0) the plane, return kAbove.  If one
  // point is above and one point is below, returns kCrossing and move the
  // above point to the plane.  If both points are below, return kBelow.
  ESide TrimAbove(CPoint3d* pt1, CPoint3d* pt2) const;
  // For completeness, we could (probably should) add a TrimBelow method, but
  // since there is no immediate need, we will defer it.
  // ESide TrimBelow(CPoint3d* pt1, CPoint3d* pt2) const;

  CPoint3d Project(const CPoint3d &pt) const;
  CUnitVector3d Project(const CUnitVector3d &v) const;
  CRay3d Project(const CRay3d &ray) const;

  bool IntersectWith(const CPoint3d &p1, const CUnitVector3d &v1,
                     CPoint3d &pt) const;
  bool IntersectWith(const CRay3d &ray, CPoint3d &pt) const;
  bool IntersectWith(const CRay3d &ray, double &t) const;
  bool IntersectWith(const CPlane &plane, CRay3d &line) const;
  int IntersectWith(const CCircle3d &circle, double &param1, double &param2,
                    CPoint3d *pP1 = 0, CPoint3d *pP2 = 0) const;
  int IntersectWith(const CCircle3d &circle, CPoint3d &pt1,
                    CPoint3d &pt2) const;

  // Test if plane is entirely above the b-box.  Used to trivially reject
  // contents of b-box relative to a clipping plane.
  bool IsAbove(const CBoundingBox3d &box) const;

  bool IsValidObj() const;

  // Returns above, below, intersecting
  // ToDo DVicknair: for consistency make IntersectWith return the ESide enum.
  int IntersectWith(const CBoundingBox3d &box) const;

  void Reverse();

  CUnitVector3d Normal() const;
  double A() const {
    return m_a;
  }
  double B() const {
    return m_b;
  }
  double C() const {
    return m_c;
  }
  double D() const {
    return m_d;
  }
  double *GetEquation(double equation[4]) const;

  // Returns point closest to origin
  CPoint3d GetPoint() const;

  // Set the plane coeficients given a point on the plane and the normal
  void Set(const CPoint3d &pt, const CUnitVector3d &normal);

  // Set the plane from coeficients
  void Set(double a, double b, double c, double d);

  // Set the "D" only ... no need to normalize
  void SetD(double d) {
    m_d = d;
  }

  // Get the canonical normal for the plane.  For all planes that
  // "equal" (ie. the same plane), this will return the same vector.
  CUnitVector3d GetCanonicalNormal() const;

  // Make sure that the plane is in "canonical" form.  If two planes
  // are the same, then their canonical form will be the same.  Basically,
  // this means that they are oriented in the same direction.
  // A plane is in "canonical" form if:
  // A^2 + B^2 + C^2 = 1 and
  // D > 0 or
  // D == 0 and C > 0 or
  // C == D == 0 and B > 0 or
  // B == C == D == 0 and A > 0
  void SetCanonicalForm();

  // Map a 3D point into a 2D point on the plane
  CPoint2d MapPoint(const CPoint3d &pt) const;

  // Offset the plane
  void operator+=(const CVector3d &vec);
  void operator-=(const CVector3d &vec);

  // Offset the plane by a signed distance along its normal
  void Offset(double offsetDistance);

  // Determine if this plane is the same as another plane
  // (within tolerance).  It is OK if the normals are in different
  // directions.
  bool Equals(const CPlane &plane, bool signedCompare = false, double tol =
              EqualTol) const;
  bool operator==(const CPlane &plane) const {
    return this->Equals(plane);
  }
  bool operator!=(const CPlane &plane) const {
    return !(this->Equals(plane));
  }

  bool Evaluate(CPoint3d *points, int nPoints);
  bool LeastSquaresFit(int nPoints, const CPoint3d *pPoints);
  bool ComputeLSF(double &sumXX, double &sumYY, double &sumZZ, double &sumWW,
                  double &sumXY, double &sumXZ, double &sumXW, double &sumYZ,
                  double &sumYW, double &sumZW);

#ifdef _CONSOLE
    void Dump() const;
#endif

 private:

  // Let the CTransformation class see the innards of this so that
  // it can transform it.
  friend class CTransformation;

  double m_a;
  double m_b;
  double m_c;
  double m_d;
};

// Useful utility class for creating a plane from a series of points.
class CPlaneCalculator {
 public:
  CPlaneCalculator();

  // add a point ... returns true if contributes to the plane definition.
  // only the first three non-colinear points contribute.
  bool AddPoint(const CPoint3d &pt);
  // returns false if no valid plane
  bool GetPlane(CPlane &plane);
  // returns number of points added so far
  int NumPoint() const {
    return m_numPoint;
  }

 private:
  CPoint3d m_p1, m_p2;
  CUnitVector3d m_v1, m_v2;
  int m_numPoint;
};
}
}
#endif  // SKETCHUP_SOURCE_COMMON_GEOMETRY_PLANE_H_
