// Copyright 2010 Google Inc. All Rights Reserved.

// Point2d.h: interface for the CPoint2d class.
//
// A 2D point with double values

#ifndef SKETCHUP_SOURCE_COMMON_GEOMETRY_POINT2D_H_
#define SKETCHUP_SOURCE_COMMON_GEOMETRY_POINT2D_H_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

//===================================================================

#include <math.h>

#include "source/common/geometry/point3d.h"
#include "source/common/geometry/tol.h"
#include "source/common/geometry/vector3d.h"

//===================================================================

namespace atlast {
namespace geometry {

class CPoint2f;
class CVector2f;

class CVector2d;

class CPoint2d {
public:
  CPoint2d() : x(0.0),
               y(0.0) {}
  CPoint2d(double _x, double _y) : x(_x),
                                   y(_y) {}
  CPoint2d(double *pt) : x(pt[0]),
                         y(pt[1]) {}
  CPoint2d(const CPoint2d &pt) : x(pt.x),
                                 y(pt.y) {}
  CPoint2d(const CPoint3d &pt) : x(pt.X()),
                                 y(pt.Y()) {}
  CPoint2d(const CPoint2f &pt);

  ~CPoint2d() {}

  CPoint2d &operator=(const CPoint2d &pt) {
    x = pt.x;
    y = pt.y;
    return *this;
  }
  CPoint2d &operator=(const CPoint3d &pt) {
    x = pt.X();
    y = pt.Y();
    return *this;
  }

  bool operator==(const CPoint2d &pt) const;
  bool operator!=(const CPoint2d &pt) const;

  //double operator* (const CPoint2d& pt) const {return x * pt.y - y * pt.x;}   // Cross product - not valid for points
  //double operator% (const CPoint2d& pt) const {return x * pt.x + y * pt.y;}   // Dot product - not valid for points

  bool EqualsWithinTol(const CPoint2d& pt, double tol) const {
    return AreEqualWithinTol(x, pt.x, tol) &&
           AreEqualWithinTol(y, pt.y, tol);
  }

  CPoint2d operator+(const CPoint2d &p) const {
    return CPoint2d(x + p.x, y + p.y);
  }
  CPoint2d operator+(const CVector2d &v) const;
  CPoint2d operator+=(const CVector2d &pt) {
    *this = *this + pt;
    return *this;
  }

  CVector2d operator-(const CPoint2d &pt) const;
  CPoint2d operator-(const CVector2d &pt) const;
  CPoint2d operator-=(const CVector2d &pt) {
    *this = *this - pt;
    return *this;
  }

  CPoint2d operator/(double d) const {
    return CPoint2d(x / d, y / d);
  }
  CPoint2d operator*(double d) const {
    return CPoint2d(x * d, y * d);
  }

  bool IsValidObj() const;

public: // data

  // Because this is a low level class, and the data is not likely
  // to change, we just make the data public.  We also just use
  // "x" and "y" rather than "m_x" for readability.

  double x;
  double y;
};

//===================================================================

inline bool CPoint2d::operator==(const CPoint2d &pt) const {
  double dx = x - pt.x;
  if (IsZero(dx) == false)
    return false;
  double dy = y - pt.y;
  if (IsZero(dy) == false)
    return false;
  return (dx * dx + dy * dy) < EqualTolSq;
}

inline bool CPoint2d::operator!=(const CPoint2d &pt) const {
  double dx = x - pt.x;
  if (IsZero(dx) == false)
    return true;
  double dy = y - pt.y;
  if (IsZero(dy) == false)
    return true;
  return (dx * dx + dy * dy) >= EqualTolSq;
}

class CVector2d {
  friend class CPoint2d;
public:
  CVector2d()
      : x(0.0),
        y(0.0) {}
  CVector2d(double _x, double _y)
      : x(_x),
        y(_y) {}
  CVector2d(double *vec)
      : x(vec[0]),
        y(vec[1]) {}
  CVector2d(const CVector2d &vec)
      : x(vec.x),
        y(vec.y) {}
  CVector2d(const CVector3d &vec)
      : x(vec.X()),
        y(vec.Y()) {}
  CVector2d(const CPoint2d &p1, const CPoint2d &p2)
      : x(p2.x - p1.x),
        y(p2.y - p1.y) {}
  CVector2d(const CVector2f &vec);

  ~CVector2d() {}

  CVector2d &operator=(const CVector2d &pt) {
    x = pt.x;
    y = pt.y;
    return *this;
  }
  CVector2d &operator=(const CVector3d &pt) {
    x = pt.X();
    y = pt.Y();
    return *this;
  }

  bool operator==(const CVector2d &pt) const;
  bool operator!=(const CVector2d &pt) const;

  double operator*(const CVector2d &pt) const {
    return x * pt.y - y * pt.x;
  }   // Cross product
  double operator%(const CVector2d &pt) const {
    return x * pt.x + y * pt.y;
  }   // Dot product

  CVector2d operator*(double s) const {
    return CVector2d(s * x, s * y);
  }

  CPoint2d operator+(const CPoint2d &pt) const {
    return CPoint2d(x + pt.x, y + pt.y);
  }
  CVector2d operator-() const {
    return CVector2d(-x, -y);
  }

  double Length() const {
    return Sqrt(x * x + y * y);
  }
  double LengthSquared() const {
    return x * x + y * y;
  }
  bool Normalize();
  bool Normalize(double &l);

 public:  // data

  // Because this is a low level class, and the data is not likely
  // to change, we just make the data public.  We also just use
  // "x" and "y" rather than "m_x" for readability.

  double x;
  double y;
};

//===================================================================

inline bool CVector2d::operator==(const CVector2d &pt) const {
  double dx = x - pt.x;
  if (IsZero(dx) == false)
    return false;
  double dy = y - pt.y;
  if (IsZero(dy) == false)
    return false;
  return (dx * dx + dy * dy) < EqualTolSq;
}

inline bool CVector2d::operator!=(const CVector2d &pt) const {
  double dx = x - pt.x;
  if (IsZero(dx) == false)
    return true;
  double dy = y - pt.y;
  if (IsZero(dy) == false)
    return true;
  return (dx * dx + dy * dy) >= EqualTolSq;
}

inline CVector2d CPoint2d::operator-(const CPoint2d &pt) const {
  return CVector2d(x - pt.x, y - pt.y);
}

inline CPoint2d CPoint2d::operator-(const CVector2d &pt) const {
  return CPoint2d(x - pt.x, y - pt.y);
}

inline bool CVector2d::Normalize() {
  double l = x * x + y * y;
  if (l < DivideByZeroTolSq)
    return false;
  l = 1.0 / Sqrt(l);
  x *= l;
  y *= l;
  return true;
}

inline bool CVector2d::Normalize(double &l) {
  l = Length();
  if (l < DivideByZeroTol)
    return false;
  double lInv = 1.0 / l;
  x *= lInv;
  y *= lInv;
  return true;
}

inline CPoint2d CPoint2d::operator+(const CVector2d &v) const {
  return CPoint2d(x + v.x, y + v.y);
}

inline CVector2d operator*(double scale, const CVector2d &v) {
  return v * scale;
}
}
}
#endif  // SKETCHUP_SOURCE_COMMON_GEOMETRY_POINT2D_H_
