// Copyright 2010 Google Inc. All Rights Reserved.

// Point3d.h: interface for the CPoint3d class.

#ifndef SKETCHUP_SOURCE_COMMON_GEOMETRY_POINT3D_H_
#define SKETCHUP_SOURCE_COMMON_GEOMETRY_POINT3D_H_

#include <iostream>
#include <vector>
#include "source/common/geometry/tol.h"

namespace atlast {
namespace geometry {

class CVector3d;
class CUnitVector3d;
class CRay3d;
class CPlane;
class CTransformation;
class CTMatrix;

//===================================================================
// Macro to make it easy to pass the coordinate into a function that
// wants 3 doubles

#define COORDS(pt)  (pt).X(),  (pt).Y(),  (pt).Z()
#define COORDSP(pt) (pt)->X(), (pt)->Y(), (pt)->Z()
#define COORDSf(pt) (float)(pt).X(),  (float)(pt).Y(),  (float)(pt).Z()
#define COORDSPf(pt) (float)(pt)->X(), (float)(pt)->Y(), (float)(pt)->Z()

//===================================================================

enum EIntType {
  kIntTypeNoIntersection  = 0,
  kIntTypeStartSeg1       = 1,
  kIntTypeEndSeg1         = 2,
  kIntTypeInteriorSeg1    = 4,
  kIntTypeStartSeg2       = 8,
  kIntTypeEndSeg2         = 16,
  kIntTypeInteriorSeg2    = 32,
  kIntTypeStart1Start2    = 9,  
  kIntTypeStart1End2      = 17,  
  kIntTypeEnd1Start2      = 10,
  kIntTypeEnd1End2        = 18,
  kIntTypeStart1Interior2 = 33,
  kIntTypeEnd1Interior2   = 34,
  kIntTypeInterior1Start2 = 12,
  kIntTypeInterior1End2   = 20,
  kIntTypeBothInterior    = 36
};
/**
 * @class CPoint3d
 * @brief A 3D point class.
 */
class CPoint3d {
 public:
  CPoint3d()
      : m_x(0.0),
        m_y(0.0),
        m_z(0.0) {}
  CPoint3d(double x, double y, double z)
      : m_x(x),
        m_y(y),
        m_z(z) {}

  CPoint3d(const double *coords)
      : m_x(coords[0]),
        m_y(coords[1]),
        m_z(coords[2]) {}

  // Use default copy ctor, dtor and assignment operator
#if 0
	CPoint3d(const CPoint3d& p)
	{
		m_x = p.m_x;
		m_y = p.m_y;
		m_z = p.m_z;
	}

	~CPoint3d() {}

	CPoint3d& operator=(const CPoint3d& p)
	{
		m_x = p.m_x;
		m_y = p.m_y;
		m_z = p.m_z;
		return *this;
	}
#endif

  /** The linear combination of two other points */
  CPoint3d(double w1, const CPoint3d &p1, double w2, const CPoint3d &p2);

  /** Point offset from another point */
  CPoint3d(const CPoint3d &pt, double dist, const CUnitVector3d &dir);

  /** Point offset from another point */
  CPoint3d(const CPoint3d &pt, const CVector3d &offset);

  bool IsValidObj() const;

  /** Perform a perspective projection */
  bool Project(double perspectiveDistance, bool bGLTransform = false);

  void SetX(double x) {
    m_x = x;
  }
  double X() const {
    return m_x;
  }

  void SetY(double y) {
    m_y = y;
  }
  double Y() const {
    return m_y;
  }

  void SetZ(double z) {
    m_z = z;
  }
  double Z() const {
    return m_z;
  }

  /** Allow access to the coordinates as if it is an array */
  double &operator[](int n) {
    return (n == 0 ? m_x : (n == 1 ? m_y : m_z));
  }

  double operator[](int n) const {
    return (n == 0 ? m_x : (n == 1 ? m_y : m_z));
  }

  // This one is a little dangerous.  It allows you to directly
  // get the coordinates of a point using the API Get methods.
  // Since the coordinates are actually saved as individual members
  // this is a little risky, but I think that is should always work
  operator double*() {
    return &m_x;
  }

  void Set(double x, double y, double z) {
    m_x = x;
    m_y = y;
    m_z = z;
  }

  void Get(double *v) const {
    v[0] = m_x;
    v[1] = m_y;
    v[2] = m_z;
  }

  void operator+=(const CVector3d &vec);
  void operator-=(const CVector3d &vec);

  CPoint3d operator+(const CVector3d &vec) const;
  CPoint3d operator+(const CPoint3d &pt) const;
  CPoint3d operator-(const CVector3d &vec) const;
  CVector3d operator-(const CPoint3d &pt) const;

  CPoint3d operator*(double scale) const {
    return CPoint3d(m_x * scale, m_y * scale, m_z * scale);
  }
  void operator*=(double scale) {
    *this = *this * scale;
  }
  CPoint3d operator/(double invScale) const {
    return *this * (1.0 / invScale);
  }
  void operator/=(double invScale) {
    *this = *this / invScale;
  }
  //CPoint3d operator* (const CTMatrix& tMatrix) const;
  
  /** Set this point to be offset from another point
   *  by a given distance in a given direction.
   * (This could be written pt = from + distance * dir.  This method is
   * provided to avoid creation of a lot of extra temporary objects.)
   */
  void Offset(const CPoint3d &from, double distance, const CUnitVector3d &dir);
  void Offset(const CPoint3d &from, const CVector3d &vec);

  /** Create a new point which is offset from this one. */
  CPoint3d Offset(double distance, const CUnitVector3d &dir) const;
  CPoint3d Offset(const CVector3d &vec) const;

  double DistanceSquaredTo(const CPoint3d &pt) const;
  double DistanceSquaredTo(const CRay3d &ray) const;
  double DistanceSquaredTo(const CPoint3d &pt, const CUnitVector3d &dir) const;

  double DistanceTo(const CPoint3d &pt) const;
  double DistanceTo(const CRay3d &ray) const;
  double DistanceTo(const CPoint3d &pt, const CUnitVector3d &dir) const;
  double DistanceTo(const CPlane &plane) const;

  bool IsOn(const CRay3d &ray) const;
  bool IsOn(const CPoint3d &pt, const CUnitVector3d &dir) const;
  bool IsOn(const CPlane &plane) const;

  CPoint3d ProjectTo(const CRay3d &ray) const;
  CPoint3d ProjectTo(const CPlane &plane) const;

  bool Equals(const CPoint3d &pt, double tol = EqualTol) const;
  bool operator==(const CPoint3d &pt) const {
    return Equals(pt);
  }
  bool operator!=(const CPoint3d &pt) const {
    return !Equals(pt);
  }

  /**  Determine which of two points is closest to this point.*/
  CPoint3d SelectClosest(const CPoint3d &p1, const CPoint3d &p2) const;

  // Compute the intersection of two lines defined by 4 points.
  // Note that if Intersect returns kIntTypeNoIntersection, s and t will be set
  // to -1.0.
  EIntType Intersect(const CPoint3d& pt1, const CPoint3d& pt2,
                     const CPoint3d& pt3, const CPoint3d& pt4,
                     double* s, double* t);

  bool Intersect(const CPoint3d& pt1, const CPoint3d& pt2,
                 const CPoint3d& pt3, const CPoint3d& pt4);

  // These methods are not really useful for doing geometric comparisons on
  // points, but they allow you to use points as a key in a sorted associative
  // container.
  bool operator<(const CPoint3d &pt) const;
  bool operator>(const CPoint3d &pt) const;
  
  // Given 3 or more planes, find the CPoint3d that is closest to ALL of
  // the planes (Least Squares Fit).
  bool LeastSquaresFit(int nPlanes, CPlane* pPlanes);
  // Same as above but using a std::vector of planes.
  bool LeastSquaresFit(const std::vector<CPlane>& planes);
 private:
  // Worker method for Least Squares Fit method.
  bool ComputeLSF(double& sumAA, double& sumAB, double& sumAC,
                  double& sumAD, double& sumBB, double& sumBC,
                  double& sumBD, double& sumCC, double& sumCD);
  // Data
  double m_x;
  double m_y;
  double m_z;

  friend class CVector3d;
  friend class CUnitVector3d;
  friend class CTransformation;
};

// This method is used when writing unit tests.
std::ostream& operator<<(std::ostream& out_stream, const CPoint3d& point);

}
}
#endif  // SKETCHUP_SOURCE_COMMON_GEOMETRY_POINT3D_H_
