// Copyright 2009 Google Inc. All Rights Reserved.

#ifndef SKETCHUP_SOURCE_COMMON_GEOMETRY_POLYGONMESH_H_
#define SKETCHUP_SOURCE_COMMON_GEOMETRY_POLYGONMESH_H_

//===================================================================

#include "source/common/geometry/point3d.h"

#pragma warning(disable:4786)
#include <vector>

namespace atlast {
namespace geometry {

class CTransformation;

// Mesh can be created with optional UV front/back coordinates,
// and optional normal vectors. UV coordinates are
// currently Point3d's with X=U, Y=V, Z=Q.  Q will always be 1
// for affine texture transforms.
#define kPolygonMeshPoints	    0
#define kPolygonMeshUVQFront	1
#define kPolygonMeshUVQBack	    2
#define kPolygonMeshNormals     4

//===================================================================
class CPolygonMesh {
public:
  CPolygonMesh(unsigned int flags=kPolygonMeshPoints);
  virtual ~CPolygonMesh();

  // Reserve space in the mesh to add points and vertices.
  // This is just an optimization to reduce the number of reallocs
  // that need to be done on the vectors.  The numbers do not have
  // to be exactly right.
  void ReserveSpace(long numPoint, long numPolygon);

  // The number of vertices (points)
  long CountPoints() const {
    return (long) m_points.size();
  }

  // Get a point. The index is 1 based because we use the sign of the
  // index within a polygon definition to flag hidden edges.
  const CPoint3d &GetPoint(long nIndex) const;
  CPoint3d &GetPoint(long nIndex);

  // Get Front/Back UV coords for index.  If coords are not stored it
  // returns 0,0,0 for each.  Note that X=U, Y=V, Z=Q, but Q is currently
  // always 0.
  CPoint3d GetUVFront(long nIndex) const;
  CPoint3d GetUVBack(long nIndex) const;

  // Set front/back UV coords for corresponding point index.  Ignored
  // if mesh was not set to store uv coords.
  void SetUVFront(long nIndex, const CPoint3d &uv);
  void SetUVBack(long nIndex, const CPoint3d &uv);

  // Get/Set normal.  If not calculated, we just return the plane normal.
  CUnitVector3d GetNormal(long nIndex) const;
  void SetNormal(long nIndex, const CUnitVector3d &normal);

  // Add a point and return its index.
  // If an existing point with the given coordinates is found, its is used.
  long InsertPoint(const CPoint3d &pt);

  // Find the index of a point with given coordinates.
  // Returns 0 is not found
  long FindPoint(const CPoint3d &pt) const;

  // The number of polygons
  long CountPolygons() const {
    return (long) m_polygons.size();
  }

  // Add a polygon.  Returns the index of the new polygon.
  long AddPolygon(long nPoint, long *indices);
  long AddPolygon(long nPoint, CPoint3d *pts);

  // Get the number of points in polygon nPoly
  // The nPoly index is 1 based to match the point indexing.
  long CountPolygonPoints(long nPoly) const;

  // Get the index of point nPoint of polygon nPoly.
  // This may return a negative value indicating that the next
  // side of the polygon is hidden.
  // The indexing for nPoly and NPoint is 1 based!
  long GetPolygonPointIndex(long nPoly, long nPoint) const;

  // Get point nPoint from polygon nPoly
  // The indexing for nPoly and NPoint is 1 based!
  CPoint3d GetPolygonPoint(long nPoly, long nPoint) const;

  // Apply a transformation to the mesh
  void Transform(const CTransformation &tform);

  // Access to flags
  unsigned int GetFlags() const {
    return m_flags;
  }

  //void Dump() const;

protected: // data

  // Points that define the vertices
  std::vector<CPoint3d> m_points;

  // Flags indicating what optional data is here
  unsigned int m_flags;

  // Optional front/back UV coords
  std::vector<CPoint3d> m_uvqFront;
  std::vector<CPoint3d> m_uvqBack;

  // Optional vertex normals.  Since our vertex normal calculation
  // assumes manifold, we always have the same normals on front/back.
  std::vector<CUnitVector3d> m_normals;

  // TODO: May need optional data like colors.
  // TODO: May want to shift over to array of doubles for efficiency.

  // This vector defines the polygons.  For each polygon, there
  // is an entry into the indices vector for where that polygon starts
  std::vector<long> m_polygons;

  // An array of indices into the m_points vector.  To find the
  // points that define a given polygon, you must first find where
  // the polygon starts in m_indices by looking it up in m_polygons.
  std::vector<long> m_indices;
};

}
}
//===================================================================
#endif  //SKETCHUP_SOURCE_COMMON_GEOMETRY_POLYGONMESH_H_
