// Copyright 2009 Google Inc. All Rights Reserved.

// Ray3d.h: interface for the CRay3d class.

#ifndef SKETCHUP_SOURCE_COMMON_GEOMETRY_RAY3D_H_
#define SKETCHUP_SOURCE_COMMON_GEOMETRY_RAY3D_H_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

//===================================================================

#include "source/common/geometry/boundingbox.h"
#include "source/common/geometry/point3d.h"
#include "source/common/geometry/unitvector3d.h"

namespace atlast {
namespace geometry {

class CVector3d;
class CPlane;
class CCircle3d;

//===================================================================

class CRay3d {
public:
  CRay3d()
      : m_start(),
        m_direction() {}
  CRay3d(const CPoint3d &start, const CUnitVector3d &direction)
      : m_start(start),
        m_direction(direction) {}
  CRay3d(const CPoint3d &start, const CVector3d &direction)
      : m_start(start),
        m_direction(direction) {}
  CRay3d(const CPoint3d &p1, const CPoint3d &p2)
      : m_start(p1),
        m_direction(p1, p2) {}

  virtual ~CRay3d() {}

  // Use default copy ctor and assignment operator
#if 0
    CRay3d(const CRay3d& ray)
        : m_start(ray.m_start), m_direction(ray.m_direction) {}

	CRay3d& operator=(const CRay3d& ray)
    {
	    m_start = ray.m_start;
	    m_direction = ray.m_direction;
	    return *this;
    }
#endif

  const CPoint3d &Start() const {
    return m_start;
  }
  const CUnitVector3d &Direction() const {
    return m_direction;
  }

  void SetStart(const CPoint3d &start) {
    m_start = start;
  }
  void SetDirection(const CUnitVector3d &direction) {
    m_direction = direction;
  }
  void SetDirection(const CVector3d &direction) {
    m_direction = direction;
  }
  void Set(const CPoint3d &start, const CUnitVector3d &direction) {
    m_start = start;
    m_direction = direction;
  }
  void Set(const CPoint3d &p1, const CPoint3d &p2);

  // Reverse the direction of the ray
  void Reverse() {
    m_direction.Reverse();
  }

  // See if two rays define the same infinite line
  bool IsOn(const CRay3d &line) const;

  // See if the ray lies on a given plane
  bool IsOn(const CPlane &plane) const;

  bool IsValidObj() const;

  // Evaluate a point on the ray at a given parameter
  CPoint3d Eval(double t) const;

  // Compute the intersection of the ray with another ray
  bool IntersectWith(const CRay3d &ray, CPoint3d &pt) const;

  // Compute the intersection of the ray with a plane.
  bool IntersectWith(const CPlane &plane, double &param) const;
  bool IntersectWith(const CPlane &plane, CPoint3d &pt) const;

  // Compute the intersection of the ray with a bounding box
  bool IntersectWith(const CBoundingBox3d &box, CPoint3d &pt,
                     EBoxSide &side) const;

  // Compute the intersection of this ray with the segment from p1 to p2
  bool IntersectWith(const CPoint3d &p1, const CPoint3d &p2,
                     double &param) const;

  // Compute the intersection(s) of this ray with the circle
  int IntersectWith(const CCircle3d &circle, double &param1, double &param2,
                    CPoint3d *pP1 = 0, CPoint3d *pP2 = 0) const;
  int IntersectWith(const CCircle3d &circle, CPoint3d &pt1,
                    CPoint3d &pt2) const;

  // Calculate a CRay3d corresponding to the intersection of 2 planes
  bool IntersectPlanes(const CPlane &plane1, const CPlane &plane2);

  // Find the projection of a point onto the ray
  CPoint3d Project(const CPoint3d &pt) const;

  // Find the points where two rays are closest.
  // pt1 is the point on this ray, pt2 is the point on the other ray.
  // Returns false if the rays are parallel.
  bool FindClosestPoints(const CRay3d &ray, CPoint3d &pt1,
                         CPoint3d &pt2) const;

  // Find t for closest point.
  double ParameterOfClosestPoint(const CPoint3d &pt) const;

  // Transform the ray
  void operator*=(const CTransformation &trans);

  friend class CTransformation;

protected:

  CPoint3d m_start;
  CUnitVector3d m_direction;

};
}
}
//===================================================================
#endif  // SKETCHUP_SOURCE_COMMON_GEOMETRY_RAY3D_H_
