// Copyright 2009 Google Inc. All Rights Reserved.

// Tol.h test for equality using a tolerance

#ifndef SKETCHUP_SOURCE_COMMON_GEOMETRY_TOL_H_
#define SKETCHUP_SOURCE_COMMON_GEOMETRY_TOL_H_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#include <math.h>

namespace atlast {
namespace geometry {

// The following functions are used for performing comparisons on floating point
// values.  The test are done using a consistent tolerance.

extern const double DivideByZeroTol;    // defined in MathUtil.cpp
extern const double DivideByZeroTolSq;  // defined in MathUtil.cpp

extern double EqualTol;   // defined in MathUtil.cpp
extern float EqualTolF;   // defined in MathUtil.cpp
extern double EqualTolSq; // defined in MathUtil.cpp
extern float EqualTolSqF; // defined in MathUtil.cpp
extern double EqualTolLatLong;  // defined in MathUtil.cpp

inline double Sqrt(double dsq) {
  return dsq > DivideByZeroTolSq ? sqrt(dsq) : 0.0;
}

// val1 == val2
inline bool EqualsWithinTol(double val1, double val2, double tol) {
  return fabs(val1 - val2) < tol;
}

inline bool Equals(double val1, double val2) {
  return EqualsWithinTol(val1, val2, EqualTol);
}

inline bool EqualsWithinPercent(double val1, double val2, double percent) {
  double diff = fabs(val1 - val2);
  double percent_diff = diff;
  // If no div by 0, then calc percent.
  if (fabs(val1) > DivideByZeroTol) {
      percent_diff = diff / val1;
      return percent_diff <= percent;
  } else { // Just do Equals();
    return diff < EqualTol;
  }
}

// val1 < val2
inline bool LessThanWithinTol(double val1, double val2, double tol) {
  return (val1 - val2) <= -tol;
}

inline bool LessThan(double val1, double val2) {
  return LessThanWithinTol(val1, val2, EqualTol);
}

// val1 > val2
inline bool GreaterThanWithinTol(double val1, double val2, double tol) {
  return (val1 - val2) >= tol;
}

inline bool GreaterThan(double val1, double val2) {
  return GreaterThanWithinTol(val1, val2, EqualTol);
}

// val1 <= val2
inline bool LessThanOrEqualWithinTol(double val1, double val2, double tol) {
  return (val1 - val2) < tol;
}

inline bool LessThanOrEqual(double val1, double val2) {
  return LessThanOrEqualWithinTol(val1, val2, EqualTol);
}

// val1 >= val2
inline bool GreaterThanOrEqualWithinTol(double val1, double val2, double tol) {
  return (val1 - val2) > -tol;
}

inline bool GreaterThanOrEqual(double val1, double val2) {
  return GreaterThanOrEqualWithinTol(val1, val2, EqualTol);
}

// val > 0
inline bool GreaterThanZeroWithinTol(double val, double tol) {
  return val >= tol;
}

inline bool GreaterThanZero(double val) {
  return val >= EqualTol;
}

// val > 0
inline bool LessThanZeroWithinTol(double val, double tol) {
  return val <= -tol;
}

inline bool LessThanZero(double val) {
  return val <= -EqualTol;
}

// Determine if two numbers are equal within a tolerance.
// Note: This looks suspiciously like EqualsWithinTol except that 
// EqualsWithinTol uses "<" and ">" comparisons whereas AreEqualWithinTol uses
// "<=" and ">="...
inline bool AreEqualWithinTol(double val1, double val2, double tol) {
  return fabs(val1 - val2) <= tol;
}

inline bool AreEqual(double val1, double val2) {
  return AreEqualWithinTol(val1, val2, EqualTol);
}

// Determine if two numbers have the same sign
inline bool SameSign(double val1, double val2) {
  return (val1 >= 0.0 && val2 >= 0.0) || (val1 < 0.0 && val2 < 0.0);
}

// Determine if a number is zero to within a tolerance
inline bool IsZeroWithinTol(double val, double tol) {
  return fabs(val) < tol;
}

inline bool IsZero(double val) {
  return IsZeroWithinTol(val, EqualTol);
}
}
}
//===================================================================
#endif  // SKETCHUP_SOURCE_COMMON_GEOMETRY_TOL_H_
