// Copyright 2009 Google Inc. All Rights Reserved.

// Transformation.h: interface for the CTransformation class.
//
// This defines a simple transformation class.  This transformation
// can represent only affine transformations (no shear)

// The transformation is stored as a partitioned 4x4 matrix
//
//      -     -
//      | R T |
//  M = | 0 w |
//      -     -
//
//  where:  M is a 4x4 matrix
//          R is a 3x3 sub-matrix. It is the rotation part
//          T is a 3x1 sub-matrix. It is the translation part
//          w is a scalar.         It is 1/scale.
//
//  The data members are:
//   m_rotation = R
//   m_translation = T
//   m_scale = 1/w
//   (Note that we are storing the scale, but need to use it as
//    1/m_scale in the calculation)
//
//////////////////////////////////////////////////////////////////////

#ifndef SKETCHUP_SOURCE_COMMON_GEOMETRY_TRANSFORMATION_H_
#define SKETCHUP_SOURCE_COMMON_GEOMETRY_TRANSFORMATION_H_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

//===================================================================
#include "source/common/geometry/boundingbox.h"
#include "source/common/geometry/plane.h"
#include "source/common/geometry/point3d.h"
#include "source/common/geometry/ray3d.h"
#include "source/common/geometry/unitvector3d.h"

namespace atlast {
namespace geometry {

class CVector3d;

//===================================================================

class CTransformation {
 public:
  CTransformation();
  CTransformation(const CUnitVector3d &x, const CUnitVector3d &y,
                  const CUnitVector3d &z, const CPoint3d &t);
  CTransformation(const CPoint3d &o, const CUnitVector3d &n);
  CTransformation(const double m[16]);

  // Use compiler generated defaults.
  //CTransformation(const CTransformation& t);
  //~CTransformation() {}
  //CTransformation& operator=(const CTransformation& t);

  bool operator==(const CTransformation &t) const;
  bool operator!=(const CTransformation &t) const;

  bool IsValidObj() const;

  // These methods modify "this" to make it into the given
  // kind of transformation.
  void Identity();
  void Translation(const CVector3d &vec);
  void Translation(const CPoint3d &pt);
  void Rotation(const CPoint3d &pt, const CUnitVector3d &vec, double angle);
  void Scaling(double scale);
  void Scaling(double sx, double sy, double sz);

  // pseudo constructors for creating transformations
  static CTransformation Translate(const CVector3d &vec);
  static CTransformation Translate(const CPoint3d &pt);
  static CTransformation Rotate(const CPoint3d &pt, const CUnitVector3d &vec,
                                double angle);
  static CTransformation Scale(double scale);
  static CTransformation Scale(double sx, double sy, double sz);
  static CTransformation Scale(const CPoint3d &about, double scale);
  static CTransformation Scale(const CPoint3d &about, double sx, double sy,
                               double sz);
  static CTransformation Inverse(const CTransformation &t);

  // Interpolate between two transformations
  static CTransformation Interpolate(const CTransformation &t1,
                                     const CTransformation &t2, double w);

  // Invert this tranform
  void Invert();

  // Apply the transform to different kinds of geometry
  void Transform(CPoint3d &pt) const;
  void Transform(CVector3d &vec) const;
  void Transform(CUnitVector3d &uvec) const;
  void Transform(CPlane &plane) const;
  // DDV - I modified Transform (CRay3d) to return the scale factor 
  // of the transformation i.e. the transformed length of a unit vector
  // in the direction of the ray.
  double Transform(CRay3d &ray) const;
  void Transform(CBoundingBox3d &box) const;
  // Transform this as a plane equation but without normalizing it.
  void Transform(double abcd[4]) const;

  CPoint3d operator*(const CPoint3d &pt) const;
  CVector3d operator*(const CVector3d &vec) const;
  CPlane operator*(const CPlane &p) const;
  CRay3d operator*(const CRay3d &ray) const;
  CBoundingBox3d operator*(const CBoundingBox3d &box) const;
  CTransformation operator*(const CTransformation &t) const;

  // Determine if this is the identity transform.  This only looks at
  // the flag to see if the transform has not been modified.  If the
  // transform has been changed, this will return false even if it is
  // really the identity.
  bool IsIdentity() const {
    return m_bIsIdentity;
  }

  // Determine if this is an identity transform by looking at the data
  bool IsReallyIdentity();

  // Check to see if the rotation part of the transform is really identity
  bool IsRotationReallyIdentity() const;

  // Test if this tranform has been mirrored.  We do this by checking
  // to see if x.Cross(y) is opposite z.  In other words, we check for
  // a left handed coordinate system.
  bool IsMirrored() const;

  // Test if this transform is orthonormal.  That means x,y,z are all at
  // right angles to each other.  Note that IsOrthogonal will return false
  // if there are any non-uniform scaling terms.
  //
  // REVIEW: If there is no scaling shouldn't IsOrthogonal return the same 
  // result here?  I (JEU) think they should be the same, but I recall 
  // IsOrthogonal returning true for matrices which had skewed axes.  
  // Revisit this.
  //
  bool IsOrthonormal() const;

  // get matrix in form usable by gl.
  // returns passed in m for convenience.
  double *GetMatrix(double m[16]) const;
  float *GetMatrixf(float m[16]) const;
  // set from gl matrix
  void SetMatrix(const double m[16]);
  void SetMatrixf(const float m[16]);

  // get components of the matrix
  CUnitVector3d X() const;
  CUnitVector3d Y() const;
  CUnitVector3d Z() const;
  CPoint3d T() const;

  // This gets UNIFORM scale only.
  double Scale() const;
  // Individual non-uniform scales are more reliable.
  double XScale() const;
  double YScale() const;
  double ZScale() const;
  // Convenient to get all three scales at once
  CVector3d XYZScale() const;

  // Z rotation relative to arbitrary axis algorithm aligned transform.
  double ZRotation() const;

  // Serialization
  double GetRotationMatrixVal(int index) const {
    return m_rotation[index];
  }
  double GetTranslationMatrixVal(int index) const {
    return m_translation[index];
  }
  double GetScale() const {
    return m_scale;
  }
  void SetTransformation(double rotation_matrix[9],
                         double translation_matrix[3], double scale);

#ifdef _CONSOLE
    void Dump() const;
#endif

 private: // methods

  void ClearRotation();
  void ClearTranslation();
  void ClearScaling();

  // Invert the 3x3 rotation part of the transform
  // (returns false if it is singular)
  bool Invert3x3();

  // Check to see if a transform that we read in has an orthogonal
  // rotation component.  If it does, adjust it if needed to improve it.
  bool ResynchOrthogonal();
  void FixOrthogonal();
 public:
  // This method is now public so we can check the orthogonal state after
  // incremental rotations on an instance.
  void CheckOrthogonal();
  // Likewise, it's useful to test if a matrix is orthogonal instead of just
  // orthonormal since orthogonal implies orthonormal and uniformly scaled.
  bool IsOrthogonal() const;

 protected: // data

  double m_rotation[9]; // a 3x3 rotation matrix
  double m_translation[3]; // the translation part
  double m_scale; // the scaling part (1/w)

  // TRANSIENT
  bool m_bIsIdentity;
  bool m_bIsOrthogonal;

};

/*
class CTMatrix : public CTransformation
{
public:
CTMatrix (double* m = NULL);
CPoint3d operator* (const CPoint3d& pt) const;
CTMatrix operator* (const CTransformation& t);
bool SetPerspective (CPoint3d eye, CPoint3d target, CVector3d up, bool p);
void ClearPerspective () {m_bPerspective = false;}
void Init (CVector3d& xVec, CVector3d& yVec, CVector3d& zVec, CPoint3d& transl);
private:
bool m_bPerspective;
double m_perspectiveFactor;
};
*/
}
}
//===================================================================
#endif  // SKETCHUP_SOURCE_COMMON_GEOMETRY_TRANSFORMATION_H_
