// Copyright 2009 Google Inc. All Rights Reserved.

// UnitVector3d.h: interface for the CUnitVector3d class.
//
//////////////////////////////////////////////////////////////////////

#ifndef SKETCHUP_SOURCE_COMMON_GEOMETRY_UNITVECTOR3D_H_
#define SKETCHUP_SOURCE_COMMON_GEOMETRY_UNITVECTOR3D_H_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

//===================================================================

#include "source/common/geometry/vector3d.h"

namespace atlast {
namespace geometry {
class CPoint3d;

//===================================================================

class CUnitVector3d {
public:
  CUnitVector3d() : m_x(0.0),
                    m_y(0.0),
                    m_z(1.0) {}
  CUnitVector3d(double x, double y, double z);
  CUnitVector3d(const double *coords);
  CUnitVector3d(const CVector3d &v);

  // Unit vector in direction from p1 to p2
  CUnitVector3d(const CPoint3d &p1, const CPoint3d &p2);

  // Use compiler generated defaults for efficiency
  //CUnitVector3d(const CUnitVector3d& v)
  //    : m_x(v.m_x), m_y(v.m_y), m_z(v.m_z) {}
  //~CUnitVector3d() {}

  CUnitVector3d &operator=(const CVector3d &v);
#if 0
	CUnitVector3d& operator=(const CUnitVector3d& v)
    {
	    m_x = v.m_x;
	    m_y = v.m_y;
	    m_z = v.m_z;
	    return *this;
    }
#endif

  double X() const {
    return m_x;
  }
  double Y() const {
    return m_y;
  }
  double Z() const {
    return m_z;
  }

  void Get(double *vals) const {
    vals[0] = m_x;
    vals[1] = m_y;
    vals[2] = m_z;
  }

  // These return the length of the vector before it was normalized.
  double Set(double x, double y, double z);
  double Set(const CPoint3d &p1, const CPoint3d &p2);

  bool IsValid() const;
  bool IsValidObj() const;

  bool Equals(const CUnitVector3d &vec, double tol = EqualTol) const;
  bool operator==(const CUnitVector3d &vec) const;
  bool operator!=(const CUnitVector3d &vec) const;
  double &operator[](int n) {
    return (n == 0 ? m_x : (n == 1 ? m_y : m_z));
  }
  double operator[](int n) const {
    return (n == 0 ? m_x : (n == 1 ? m_y : m_z));
  }
  bool IsParallelTo(const CUnitVector3d &vec) const;
  bool IsPerpendicularTo(const CUnitVector3d &vec) const;
  bool IsSameDirectionAs(const CUnitVector3d &vec) const;

  // Operations

  void Reverse();
  CUnitVector3d operator-() const;
  CUnitVector3d operator+(const CUnitVector3d &vec) const;
  CUnitVector3d operator-(const CUnitVector3d &vec) const;
  CVector3d operator*(const CVector3d &vec) const; // cross product
  CVector3d operator*(const CUnitVector3d &vec) const; // cross product
  CVector3d Cross(const CVector3d &vec) const;
  CVector3d Cross(const CUnitVector3d &vec) const;
  double operator%(const CVector3d &vec) const; // dot product
  double operator%(const CUnitVector3d &vec) const; // dot product
  double Dot(const CVector3d &vec) const;
  double Dot(const CUnitVector3d &vec) const;
  CVector3d operator*(double scale) const;
  friend CVector3d operator*(double sclae, const CUnitVector3d &vec);

  // Get the angle between this unit vector and another one.
  // The returned angle is between 0 and PI radians
  double AngleBetween(const CUnitVector3d &uvec) const;

  // Compute the angle between the projection of two vectors onto
  // the plane perpendicular to this vector measured CCW.
  // The result will be between 0 and 2PI
  double CCWAngleBetween(const CVector3d &v1, const CVector3d &v2) const;

  // Get arbitrary axes perpendicular to this vector.
  // This method assumes that this vector will be the Z axis of a
  // coordinate system, and computes the X and Y axes.
  // The results are returned in xaxis and yaxis.
  void GetArbitraryAxes(CUnitVector3d &xaxis, CUnitVector3d &yaxis) const;

private:

  bool Normalize();
  // Returns the length before it was normalized
  bool Normalize(double &l);

  double m_x;
  double m_y;
  double m_z;

  friend class CPoint3d;
  friend class CVector3d;
  friend class CTransformation;
  friend class CPlane;
};

//===================================================================
// CVector inlines that are defined here rather than in Vector3d.h
// because they require the definition of CUnitVector3d.  Presumable
// Anyone who will use one of these is already including CUnitVector3d
// because they are using it.

inline CVector3d::CVector3d(const CUnitVector3d &v)
    : m_x(v.m_x),
      m_y(v.m_y),
      m_z(v.m_z) {}

inline CVector3d &CVector3d::operator=(const CUnitVector3d &v) {
  m_x = v.m_x;
  m_y = v.m_y;
  m_z = v.m_z;
  return *this;
}

// dot product
inline double CVector3d::Dot(const CUnitVector3d &vec) const {
  return (m_x * vec.m_x) + (m_y * vec.m_y) + (m_z * vec.m_z);
}

inline double CVector3d::operator%(const CUnitVector3d &vec) const {
  return (m_x * vec.m_x) + (m_y * vec.m_y) + (m_z * vec.m_z);
}


//===================================================================
// Methods implemented inline as a result of profiling.

// dot product
inline double CUnitVector3d::operator%(const CVector3d &vec) const {
  return (m_x * vec.m_x) + (m_y * vec.m_y) + (m_z * vec.m_z);
}

inline double CUnitVector3d::operator%(const CUnitVector3d &vec) const {
  return (m_x * vec.m_x) + (m_y * vec.m_y) + (m_z * vec.m_z);
}

// dot product
inline double CUnitVector3d::Dot(const CVector3d &vec) const {
  return (m_x * vec.m_x) + (m_y * vec.m_y) + (m_z * vec.m_z);
}

// dot product
inline double CUnitVector3d::Dot(const CUnitVector3d &vec) const {
  return (m_x * vec.m_x) + (m_y * vec.m_y) + (m_z * vec.m_z);
}
}
}
//===================================================================
#endif  // SKETCHUP_SOURCE_COMMON_GEOMETRY_UNITVECTOR3D_H_
