// Copyright 2009 Google Inc. All Rights Reserved.

// Vector3d.h: interface for the CVector3d class.
//
//////////////////////////////////////////////////////////////////////

#ifndef SKETCHUP_SOURCE_COMMON_GEOMETRY_VECTOR3D_H_
#define SKETCHUP_SOURCE_COMMON_GEOMETRY_VECTOR3D_H_

//===================================================================

#include <math.h>
#include <iostream>
#include "source/common/geometry/point3d.h"

namespace atlast {
namespace geometry {
class CUnitVector3d;
class CTransformation;

//===================================================================

class CVector3d {
public:
  CVector3d()
      : m_x(0.0),
        m_y(0.0),
        m_z(0.0) {}
  CVector3d(double x, double y, double z)
      : m_x(x),
        m_y(y),
        m_z(z) {}
  CVector3d(const double *coords)
      : m_x(coords[0]),
        m_y(coords[1]),
        m_z(coords[2]) {}

  // Use compiler generated defaults for efficiency.
  // Copy constructor
  //CVector3d (const CVector3d& v) {m_x = v.m_x; m_y = v.m_y; m_z = v.m_z;}
  //~CVector3d() {}
  //CVector3d& operator= (const CVector3d& v) {m_x = v.m_x; m_y = v.m_y; m_z = v.m_z; return *this;}

  // COnvert a unit vector to a vector
  CVector3d(const CUnitVector3d &v);

  // Vector from a length and a direction
  CVector3d(double length, const CUnitVector3d &dir);

  // Vector in direction from p1 to p2
  CVector3d(const CPoint3d &p1, const CPoint3d &p2);

  // Linear combination of two vectors
  CVector3d(double w1, const CUnitVector3d &v1, double w2,
            const CUnitVector3d &v2);

  // Create a vector from 3 axis vectors
  CVector3d(double x, const CUnitVector3d &xaxis, double y,
            const CUnitVector3d &yaxis, double z, const CUnitVector3d &zaxis);

  CVector3d &operator=(const CUnitVector3d &v);

  void SetX(double x) {
    m_x = x;
  }
  double X() const {
    return m_x;
  }

  void SetY(double y) {
    m_y = y;
  }
  double Y() const {
    return m_y;
  }

  void SetZ(double z) {
    m_z = z;
  }
  double Z() const {
    return m_z;
  }

  void Set(double x, double y, double z) {
    m_x = x;
    m_y = y;
    m_z = z;
  }
  void Get(double *vals) const {
    vals[0] = m_x;
    vals[1] = m_y;
    vals[2] = m_z;
  }

  // Method brought here from UnitVector3d so we can do similar IsValid tests
  // after cross product without taking normalize hit.
  bool IsValid() const {
    return m_x != 0.0 || m_y != 0.0 || m_z != 0.0;
  }
  bool IsValidObj() const;

  double Length() const {
    return Sqrt(m_x * m_x + m_y * m_y + m_z * m_z);
  }
  double LengthSquared() const {
    return m_x * m_x + m_y * m_y + m_z * m_z;
  }
  bool SetLength(double length);
  bool Normalize();
  bool Normalize(double &l);
  void Reverse();

  CVector3d operator+(const CVector3d &vec) const;
  CVector3d operator-(const CVector3d &vec) const;
  CVector3d operator*(double scale) const;
  CVector3d operator/(double scale) const;
  friend CVector3d operator*(double scale, const CVector3d &vec);
  CVector3d operator-() const;
  CVector3d operator*(const CVector3d &vec) const; // cross product
  CVector3d operator*(const CUnitVector3d &vec) const; // cross product
  CVector3d Cross(const CVector3d &vec) const;
  CVector3d Cross(const CUnitVector3d &vec) const;
  double operator%(const CVector3d &vec) const; // dot product
  double operator%(const CUnitVector3d &vec) const; // dot product
  double Dot(const CVector3d &vec) const;
  double Dot(const CUnitVector3d &vec) const;
  void operator+=(const CVector3d &vec);
  void operator-=(const CVector3d &vec);
  void operator*=(double scale);
  void operator/=(double scale);
  bool operator==(const CVector3d &vec) const;
  bool operator!=(const CVector3d &vec) const;

  double &operator[](int n) {
    return (n == 0 ? m_x : (n == 1 ? m_y : m_z));
  }
  double operator[](int n) const {
    return (n == 0 ? m_x : (n == 1 ? m_y : m_z));
  }

  // See comment in Point3d.h for this operator
  operator double*() {
    return &m_x;
  }

  // Get the angle between this unit vector and another one.
  // Neither vector can be zero length.
  // The returned angle is between 0 and PI radians
  double AngleBetween(const CVector3d &uvec) const;

  // This method is not really useful for doing geometric comparisons on points,
  // but it allows you to put points into a set.
  bool operator<(const CVector3d &pt);

private:
  double m_x;
  double m_y;
  double m_z;


  friend class CPoint3d;
  friend class CUnitVector3d;
  friend class CTransformation;
};

//===================================================================
// Methods implemented inline as a result of profiling.

// Vector in direction from p1 to p2
inline CVector3d::CVector3d(const CPoint3d &p1, const CPoint3d &p2)
    : m_x(p2.m_x - p1.m_x),
      m_y(p2.m_y - p1.m_y),
      m_z(p2.m_z - p1.m_z) {}

// dot product
inline double CVector3d::operator%(const CVector3d &vec) const {
  return (m_x * vec.m_x) + (m_y * vec.m_y) + (m_z * vec.m_z);
}

// dot product
inline double CVector3d::Dot(const CVector3d &vec) const {
  return (m_x * vec.m_x) + (m_y * vec.m_y) + (m_z * vec.m_z);
}

// This method is used when writing unit tests.
std::ostream& operator<<(std::ostream& out_stream, const CVector3d& vector);

}
}
//===================================================================
#endif  // SKETCHUP_SOURCE_COMMON_GEOMETRY_VECTOR3D_H_
