// Copyright 2010 Google Inc. All Rights Reserved.

// This defines a special allocator that can be used in STL collections.
// The purpose of this special allocator is to prevent the problem that
// can occur when you create a collection in SketchUp and then modify it
// in a DLL, then the standard allocator can cause a crash because the
// memory was allocated in one EXE/DLL and freed in a different one.

// This implementation just uses a function that wraps new and delete.  The
// function lives in the right place, so it should always allocate memory
// from the same source.

#ifndef SKETCHUP_SOURCE_COMMON_UTILS_ALLOCATOR_H_
#define SKETCHUP_SOURCE_COMMON_UTILS_ALLOCATOR_H_

#include <stddef.h>
#include <limits>
#include <iostream>

#include "source/common/utils/alutils.h"

//=============================================================================

// These functions do the actual allocation and deallocation.  They are just
// wrappers for new and delete
extern UTILS_EXPORT void *atlast_utils_allocate(size_t num);
extern UTILS_EXPORT void atlast_utils_deallocate(void *ptr);

//=============================================================================

namespace atlast {
namespace utils {

template <class T>
class allocator {
 public:
  // type definitions
  typedef T value_type;
  typedef T *pointer;
  typedef const T *const_pointer;
  typedef T &reference;
  typedef const T &const_reference;
  typedef size_t size_type;
  typedef ptrdiff_t difference_type;

  // rebind allocator to type U
  template <class U>
struct rebind {
  typedef allocator<U> other;
};

  // return address of values
  pointer address(reference value) const {
    return &value;
  }

  const_pointer address(const_reference value) const {
    return &value;
  }

  // The allocator doesn't keep any state, so the constructors and destructors
  // don't need to do anything.
  allocator()
  throw() {}
  allocator(const allocator&)
  throw() {}
#if defined(__APPLE__) || defined(__LINUX__)
    // This doesn't compile on Windows.  I don't think it is needed
    template <class U>
        allocator (const allocator<U>&) throw() { }
#endif
  ~allocator()
  throw() {}

  // return maximum number of elements that can be allocated
  size_type max_size() const
  throw() {
#ifdef _WINDOWS
    // I don't know why I get an error on numeric_limits::max()
    return INT_MAX / sizeof(T);
#else
        return std::numeric_limits<size_t>::max() / sizeof(T);
#endif
  }

  // initialize elements of allocated storage p with value value
  void construct(pointer p, const T &value) {
    // initialize memory with placement new
    new((void*) p)T(value);
  }

  // destroy elements of initialized storage p
  void destroy(pointer p) {
    // destroy objects by calling their destructor
    p->~T();
  }

  // The allocate and deallocate methods are the only ones that really
  // do anything interesting.

  // allocate but don't initialize num elements of type T
  pointer allocate(size_type num, const void* = 0) {
#ifndef ALAppKit
    return (pointer)atlast_utils_allocate(num * sizeof(T));
#else
    return (pointer)NULL;
#endif
  }

  // deallocate storage p of deleted elements
  void deallocate(pointer p, size_type) {
#ifndef ALAppKit
    atlast_utils_deallocate((void*) p);
#endif
  }
};

// return that all specializations of this allocator are interchangeable
template <class T1, class T2>
bool operator==(const allocator<T1>&, const allocator<T2>&)
throw() {
  return true;
}

template <class T1, class T2>
bool operator!=(const allocator<T1>&, const allocator<T2>&)
throw() {
  return false;
}
}
}  // end of namespaces

#endif  // SKETCHUP_SOURCE_COMMON_UTILS_ALLOCATOR_H_
