// Copyright 2010 Google Inc. All Rights Reserved.
#ifndef SKETCHUP_SOURCE_COMMON_UTILS_COLOR_H_
#define SKETCHUP_SOURCE_COMMON_UTILS_COLOR_H_

#if _MSC_VER > 1000
#pragma once
#endif  // _MSC_VER > 1000

#ifdef _WINDOWS
#include "windows.h"
#endif  // _WINDOWS
//===================================================================

#if defined(__APPLE__) || defined(__LINUX__)
#include "source/common/utils/mac/wintype.h"
#include "source/common/utils/mac/tchar.h"
typedef unsigned long COLORREF;
#define RGB(r,g,b) ((COLORREF)(((BYTE)(r)|((WORD)((BYTE)(g))<<8))|(((DWORD)(BYTE)(b))<<16)))
#define GetRValue(rgb) ((BYTE)(rgb))
#define GetGValue(rgb) ((BYTE)(((WORD)(rgb)) >> 8))
#define GetBValue(rgb) ((BYTE)((rgb)>>16))
#endif

#define RGBA(r,g,b,a) ((COLORREF)((((BYTE)(r)|((WORD)((BYTE)(g))<<8))|(((DWORD)(BYTE)(b))<<16))|(((DWORD)(BYTE)(a))<<24)))
#define GetAValue(rgba) ((BYTE)((rgba)>>24))

#define RGB_MASK    0x00ffffff
#ifdef ALPHA_MASK
#undef ALPHA_MASK
#endif
#define ALPHA_MASK  0xff000000

struct RGBA {
  BYTE r,g,b,a;
};

namespace atlast {
namespace utils {
class CColor {
 public:
  CColor() : m_color(0) {}
  CColor(BYTE red, BYTE green, BYTE blue, BYTE alpha = 255);
  CColor(double red, double green, double blue, double alpha = 1.0);
  CColor(COLORREF color) {
    m_color = color/* | ALPHA_MASK*/;
  }

  // Use default assignment operator, copy ctor, and dtor.
  //CColor& operator=(const CColor& color);
  //CColor(const CColor& color) : m_color(color.m_color) {}
  //~CColor() {}

  CColor &operator=(COLORREF color);

  bool operator==(const CColor &color) const {
    return m_color == color.m_color;
  }
  bool operator==(COLORREF color) const;
  bool operator!=(const CColor &color) const {
    return m_color != color.m_color;
  }
  bool operator!=(COLORREF color) const;
  bool operator<(const CColor& color) const {
    return m_color < color.m_color;
  }

  BYTE GetRed() const {
    return GetRValue(m_color);
  }
  BYTE GetGreen() const {
    return GetGValue(m_color);
  }
  BYTE GetBlue() const {
    return GetBValue(m_color);
  }
  BYTE GetAlpha() const {
    return GetAValue(m_color);
  }

  // See if the color is set
  bool IsSet() const {
    return m_color != 0;
  }

  float GetRedf() const;
  float GetGreenf() const;
  float GetBluef() const;
  float GetAlphaf() const;

  // Get color in form useful for glMaterialFv.  Pass in your own array
  // or NULL to reuse static one.  Return value can be passed directly
  // to glMaterialFv.
  float *GetFv(float rgba[4]= NULL) const;

  // Opaque
  COLORREF GetOpaqueColor() const {
    return m_color | ALPHA_MASK;
  }

  void SetOpaqueColor(COLORREF color) {
    m_color = color | ALPHA_MASK;
  }

  // Mfc controls use RGB color.  This is not the same as opaque, so we want
  // separate methods to set/get the values.
  COLORREF GetRGB() const;
  void SetRGB(COLORREF rgb);

  // RGBA color
  COLORREF GetRGBA() const {
    return m_color;
  }

  void SetRGBA(COLORREF rgba) {
    m_color = rgba;
  }

  // hls
  bool GetHLS(double &h, double &l, double &s) const;
  bool SetHLS(double h, double l, double s);

  // convert hls to rgb and back
  static void RGBToHLS(BYTE &rb, BYTE &gb, BYTE &bb, double &h, double &l,
                       double &s);

  static void HLSToRGB(double &h, double &l, double &s, BYTE &rb, BYTE &gb,
                       BYTE &bb);

  // hsv
  bool GetHSV(double &h, double &s, double &v) const;
  bool SetHSV(double h, double s, double v);

  // convert hsv to rgb and back
  static void RGBToHSV(BYTE &rb, BYTE &gb, BYTE &bb, double &h, double &s,
                       double &v);

  static void HSVToRGB(double &h, double &s, double &v, BYTE &rb, BYTE &gb,
                       BYTE &bb);

  // Get the brightness of this color
  float GetBrightness() const;

  // Get the luminence of this color
  double GetLuminence() const;

  // Convert this color into a shade of gray with the same luminence
  void MakeGray();
  bool IsMonochrome() const;

  void Set(BYTE red, BYTE green, BYTE blue, BYTE alpha = 255);
  void Set(double red, double green, double blue, double alpha = 1.0);

  void SetAlpha(BYTE alpha);
  void SetAlpha(double alpha);

  void ScaleRGB(double scale);

  // Blend this color with another color.
  // 0 <= w <= 1
  // The result is w * (*this) + (1-w) * color
  void Blend(const CColor &color, double w = .5);
  static CColor Blend(const CColor &c1, const CColor &c2, double w = 0.5);

  // Get a name for this color.  Returns a static string
  const char *GetName() const;

  // Set the color from a name.  Returns false if the name is not found
  bool SetFromName(LPCTSTR name);

  // Blend this color into a background color
  void BlendInto(const CColor* background);

  // These methods let you find what color names are available
  static int NumberOfColorNames();
  static const char *GetColorName(int index);

  // Generates a random color.
  static CColor Random();

  // Comparisons
  static bool LessThan(double val1, double val2);
  static bool GreaterThan(double val1, double val2);
  static bool Equals(double val1, double val2);
  static bool LessThanOrEqual(double val1, double val2);
  static double Clamp(double val, double min, double max);

  // Some standard colors
  static const CColor Red;
  static const CColor Green;
  static const CColor Blue;
  static const CColor Yellow;
  static const CColor Cyan;
  static const CColor Magenta;
  static const CColor DarkRed;
  static const CColor DarkGreen;
  static const CColor DarkBlue;
  static const CColor DarkYellow;
  static const CColor DarkCyan;
  static const CColor DarkMagenta;
  static const CColor Orange;
  static const CColor White;
  static const CColor Black;
  static const CColor Gray;
  static const CColor LightGray;
  static const CColor DarkGray;

  // A special color indicating that no color is set.
  static const CColor NoColor;

 private:

  // To make debugging colors WAY simpler on the PC, add the following line
  // to the end of AutoExp.dat in your dev studio directory.  Then when you
  // hover over a variable it will pop up this useful output.
  //
  //CColor=r=<m_rgba.r,u> (<m_rgba.r,x>) g=<m_rgba.g,u> (<m_rgba.g,x>) b=<m_rgba.b,u> (<m_rgba.b,x>) a=<m_rgba.a,u> (<m_rgba.a,x>)
#if defined(__APPLE__) || !defined(_DEBUG)
  COLORREF m_color;
#else
  union {
    COLORREF m_color;
    RGBA m_rgba;
  };
#endif
};
}  // namespace utils
}  // namespace atlast

//===================================================================

//-------------------------------------------------------------------
// Convert BYTE to Double and back
//-------------------------------------------------------------------
inline BYTE Double01ToByte(double val) {
  if (val <= 0.0) {
    return 0;
  } else if (val >= 1.0) {
    return 255;
  } else {
    return (BYTE) (val * 255.0);
  }
}

inline float ByteToFloat(BYTE val) {
  return (float) val / 255.0f;
}

//===================================================================
#endif  // SKETCHUP_SOURCE_COMMON_UTILS_COLOR_H_
