// Copyright 2010 Google Inc. All Rights Reserved.

// A string of unicode character.  This is the basic string class used
// in PinUp.

// Enforce _GLIBCXX_FULLY_DYNAMIC_STRING
#if defined(__APPLE__) || defined(__LINUX__)
#ifdef _BASIC_STRING_H
#ifndef _GLIBCXX_FULLY_DYNAMIC_STRING
#error including <basic_string.h> with _GLIBCXX_FULLY_DYNAMIC_STRING NOT defined!
#endif
#endif
#endif

#ifndef SKETCHUP_SOURCE_COMMON_UTILS_USTRING_H_
#define SKETCHUP_SOURCE_COMMON_UTILS_USTRING_H_

#include <string>

#include "source/common/utils/allocator.h"
#include "source/common/utils/alutils.h"
#include "source/common/utils/unichar.h"

#ifdef _WINDOWS
#include "tchar.h"
#endif // _WINDOWS_

#if defined(__APPLE__) || defined(__LINUX__)
#include "source/common/utils/ustring_traits.h"
#endif

// SAFE_COPY can be used on ustring and std::string to do a safe copy call
// on windows and the standard copy call on linux and mac.
// Usage::
// my_string.SAFE_COPY(destination_pointer, destination_size, copy_count);
#ifdef _WINDOWS
#define SAFE_COPY(destination, destination_size, copy_count)\
  _Copy_s((destination), (destination_size), (copy_count));
#else
#define SAFE_COPY(destination, destination_size, copy_count)\
  copy((destination), (copy_count));
#endif

namespace atlast {
namespace utils {

//===================================================================
// We cannot really use the standard implementation of wstring.  
// The problem is that it is defined as a string of wchar_t.  Unfortunately,
// on the Mac wchar_t is defined as a 32 bit value.  Everything that works
// with unicode though wants 16 bit values - i.e. UNICHAR.

#if defined(__APPLE__) || defined(__LINUX__)
typedef std::basic_string<UNICHAR, std::char_traits<UNICHAR>, atlast::utils::allocator<UNICHAR> > ustring_base;
#else
typedef std::basic_string<UNICHAR, std::char_traits<wchar_t>,
  atlast::utils::allocator<UNICHAR> > ustring_base;
#endif

class UTILS_EXPORT atlast_ustring : public ustring_base {
public:
  // These are the standard constructors.  Unfortunately they are not inherited.
  atlast_ustring() : ustring_base() {}
  atlast_ustring(const ustring_base &s) : ustring_base(s) {}
  atlast_ustring(const atlast_ustring &s) : ustring_base(s) {}
  atlast_ustring(const UNICHAR *s) : ustring_base() {
    if (s)
      *this = ustring_base(s);
  }
  atlast_ustring(size_type num, UNICHAR c) : ustring_base(num, c) {}

  // The constructor that takes a simple C string assumes that the string
  // uses utf-8 encoding.  In almost all cases that use this, the string will
  // actually just be a simple ASCII string
  atlast_ustring(const char *s);
  atlast_ustring(const char *s, unsigned long length);

  // This is equivalent to the version that takes a const char*.  It just gets
  // the string from the std::string using c_str()
  atlast_ustring(const std::string &s);

  atlast_ustring &operator=(const char *s);
  atlast_ustring &operator=(const std::string &s);
#ifndef _WINDOWS
    atlast_ustring& operator=(const std::wstring& s);
#endif
  void assign_utf8(const char *s);
  void assign_utf8(const char *s, unsigned long numchar8);

  // Get a utf8 string from this string
  std::string utf8() const;

  size_t size_utf8() const;

  // Get a wstring.
  std::wstring w_str() const;

  // Compare with standard C strings.  This compares the utf8 representation
  // of the string
  bool operator==(const char *s) const;
  bool operator!=(const char *s) const;

  // Remove whitespace from beginning, end or both
  // The non-const versions modify the string itself
  atlast_ustring &trim_left();
  atlast_ustring trim_left() const;
  atlast_ustring &trim_right();
  atlast_ustring trim_right() const;
  atlast_ustring &trim();
  atlast_ustring trim() const;

  // Replaces the 'old_string' with 'new_string'
  // Returns the number of replaced instances of the character or substring. 
  // Zero if the string is not changed.
  // Equivalent to CString::Replace.
  int replace_all(const atlast_ustring &old_string,
                  const atlast_ustring &new_string);

  // Convert to upper or lower case
  void toupper();
  void tolower();

  int compare_nocase(const atlast_ustring &s) const;

  //void vformat(const UNICHAR* fmt, va_list args );
  //void format(const UNICHAR* fmt, ... );

  void FormatV(const UNICHAR *pszFormat, va_list args);
  void Format(const UNICHAR *pszFormat, ...);

  void Format(const char *pszFormat, ...);

  // CString support.  Leaving this in to help convert
  // CString calls to ustring.
  // TODO: Joe doesn't want any CString functions in here, need to remove,
  // but need to convert tons of code first.
  atlast_ustring Left(int num) const {
    return substr(0, num);
  }
  atlast_ustring Mid(int index) const {
    return substr(index, length() - index);
  }
  atlast_ustring Mid(int index, int num) const {
    return substr(index, num);
  }
  atlast_ustring Right(int num) const {
    return substr(length() - num, num);
  }
};
}  // namespace atlast
}  // namespace utils {

typedef atlast::utils::atlast_ustring AtLastUstring;

inline AtLastUstring operator+(const char *s1, const AtLastUstring &s2) {
  return AtLastUstring(s1) + s2;
}

inline AtLastUstring operator+(const AtLastUstring &s1, const char *s2) {
  return s1 + AtLastUstring(s2);
}

// output operator for ustring
template <class charT, class traits>
inline
    std::basic_ostream<charT, traits> &operator<<(std::basic_ostream<charT,
                                                  traits> &strm,
                                                  const AtLastUstring &s) {
  // REVIEW: I could do escape processing to show unicode chars as something like
  // uxxxx instead of doing utf8.
  return strm << s.utf8();
}

// REVIEW: By using UNICHAR instead of wchar_t, we lose all of the 
// automatic formatting from the C++ standard library.  I.e. you can't
// use wstringstream.  I could create a new kind of ustringstream as
// a std::basic_stringstream<UNICHAR>, but then I'd have to implement all
// of the formatting methods.  Maybe I should just use a standard wstring
// in PinUp and provide easy conversion to/from NSString on the Mac.

#endif  // SKETCHUP_SOURCE_COMMON_UTILS_USTRING_H_
