// Copyright 2010 Google Inc. All Rights Reserved.
#ifndef SKETCHUP_SOURCE_SKETCHUP_SKPWRITER_SAPI_DEFS_H_
#define SKETCHUP_SOURCE_SKETCHUP_SKPWRITER_SAPI_DEFS_H_

#if _MSC_VER > 1000
#pragma once
#endif  // _MSC_VER > 1000

#include <set>
#include "source/sketchup/skpwriter/sapi/ientity.h"

namespace atlast {
namespace sketchup {

/**
 * @interface ISketchUpCollection
 */
class ISketchUpCollection {
 public:
  virtual ~ISketchUpCollection() {}
  /**
   * @brief Adds the given entity to the collection, if appropriate.
   * This method is responsible for memory management (AddRef and Release).
   *
   * @param pEnt The object to add to the collection.
   */
  virtual void Add(ISketchUpEntity *pEnt) = 0;
};
////////////////////////////////////////////////////////////////////////////////////
/**
 * @interface ISketchUpEntitySetFilter
 */
class ISketchUpEntitySetFilter {
 public:
  virtual ~ISketchUpEntitySetFilter() {}
  /**
   * @brief Checks to see whether the given entity is accepted by the filter.
   * @param pEnt The entity to check.
   * @return true iff the object passes the filter's test(s).
   */
  virtual bool Accept(ISketchUpEntity *pEnt) const = 0;
};
////////////////////////////////////////////////////////////////////////////////////
/**
 * @class SketchUpEntitySet
 */
class SketchUpEntitySet {
 public:
  SketchUpEntitySet();
  SketchUpEntitySet(const SketchUpEntitySet &other);
  virtual ~SketchUpEntitySet();

  SketchUpEntitySet &operator=(const SketchUpEntitySet &other);

  void Add(ISketchUpEntity *pObj);

  void Get(const ISketchUpEntitySetFilter *filter,
           ISketchUpCollection *collection) const;

  int GetCount() const;

private: //  methods
  void CopyFrom(const SketchUpEntitySet &other);

private: // data
  typedef std::set<ISketchUpEntity*> SkpEntitySet;
  SkpEntitySet m_entities;
};
////////////////////////////////////////////////////////////////////////////////////
/**
 * @class AcceptAllFilter
 * @brief A filter that allows everything to pass through.
 */
class AcceptAllFilter : public ISketchUpEntitySetFilter {
 public:
  virtual ~AcceptAllFilter() {}
  /**
   * @brief Checks to see whether the given entity is accepted by the filter.
   * In this case, everything is accepted.
   * @param pEnt The entity to check.
   * @return true iff the object passes the filter's test(s).
   */
  virtual bool Accept(ISketchUpEntity *pEnt) const {
    return true;
  }
};

/**
 * @class SketchUpEntityCollection
 * @brief A collection of entities.
 */
class SketchUpEntityCollection : public ISketchUpCollection {
 public:
  virtual ~SketchUpEntityCollection() {
    for (unsigned int i = 0; i < m_ents.size(); i++) {
      m_ents[i]->Release();
    }
  }
  /**
   * @brief Adds the given entity to the collection, if appropriate.
   * This method is responsible for memory management (AddRef and Release).
   *
   * @param pEnt The object to add to the collection.
   */
  virtual void Add(ISketchUpEntity *pEnt) {
    if (pEnt != NULL) {
      pEnt->AddRef();
      m_ents.push_back(pEnt);
    }
  }
 public: // data is public for simplicity
  std::vector<ISketchUpEntity*> m_ents;
};
////////////////////////////////////////////////////////////////////////////////////
template<class T>
class SketchUpTypedFilter : public ISketchUpEntitySetFilter {
 public:
  virtual ~SketchUpTypedFilter() {}
  virtual bool Accept(ISketchUpEntity *pObj) const {
    return (dynamic_cast<T*>(pObj) != NULL);
  }
};

template<class T>
class SketchUpTypedCollection : public ISketchUpCollection {
 public:
  virtual ~SketchUpTypedCollection() {
    for (unsigned int i = 0; i < m_objs.size(); i++) {
      m_objs[i]->Release();
    }
  }
  T *Get(int i) {
    if (i < 0 || i >= GetCount()) {
      return NULL;
    }
    m_objs[i]->AddRef();
    return m_objs[i];
  }

  int GetCount() const {
    return (int) m_objs.size();
  }

  virtual void Add(ISketchUpEntity *pEnt) {
    T *pType = dynamic_cast<T*>(pEnt);
    if (pType != NULL) {
      pType->AddRef();
      m_objs.push_back(pType);
    }
  }

private:
  std::vector<T*> m_objs;
};
#define DECLARE_STANDARD_FILTER(_type, _name)                               \
    typedef SketchUpTypedFilter<_type> SketchUp##_name##Filter;             \
    typedef SketchUpTypedCollection<_type> SketchUp##_name##Collection;     \

////////////////////////////////////////////////////////////////////////////////////
#define DECLARE_STANDARD_FILTER2(_type, _name)                              \
class SketchUp##_name##Filter : public ISketchUpEntitySetFilter             \
{                                                                           \
public:                                                                     \
    virtual bool Accept(ISketchUpEntity* pObj) const                        \
    {                                                                       \
        return (dynamic_cast<_type*>(pObj) != NULL);                        \
    }                                                                       \
};                                                                          \
class SketchUp##_name##Collection : public ISketchUpCollection              \
{                                                                           \
public:                                                                     \
    virtual ~SketchUp##_name##Collection()                                  \
    {                                                                       \
        for(unsigned int i=0;i<m_objs.size(); i++)                          \
        {                                                                   \
            m_objs[i]->Release();                                           \
        }                                                                   \
    }                                                                       \
    _type* Get(int i)                                                       \
    {                                                                       \
        if(i < 0 || i >= GetCount())                                        \
        {                                                                   \
            return NULL;                                                    \
        }                                                                   \
        m_objs[i]->AddRef();                                                \
        return m_objs[i];                                                   \
    }                                                                       \
                                                                            \
    int GetCount() const                                                    \
    {                                                                       \
        return (int)m_objs.size();                                          \
    }                                                                       \
                                                                            \
    virtual void Add(ISketchUpEntity* pEnt)                                 \
    {                                                                       \
        _type* pType = dynamic_cast<_type*>(pEnt);                          \
        if(pType!= NULL)                                                    \
        {                                                                   \
            pType->AddRef();                                                \
            m_objs.push_back(pType);                                        \
        }                                                                   \
    }                                                                       \
                                                                            \
private:                                                                    \
    std::vector<_type*> m_objs;                                             \
};                                                                          \
//////////////////// IMPLEMENTATION Do Not Look Past Here ////////////////////

inline SketchUpEntitySet::SketchUpEntitySet() {}

inline SketchUpEntitySet::SketchUpEntitySet(const SketchUpEntitySet &other) {
  CopyFrom(other);
}

inline SketchUpEntitySet &SketchUpEntitySet::operator=(const SketchUpEntitySet
                                                       &other) {
  CopyFrom(other);
  return *this;
}

inline SketchUpEntitySet::~SketchUpEntitySet() {
  for (SkpEntitySet::const_iterator it = m_entities.begin();
       it != m_entities.end();
       it++) {
    if (*it != NULL) {
      (*it)->Release();
    }
  }
}
inline void SketchUpEntitySet::CopyFrom(const SketchUpEntitySet &other) {
  for (SkpEntitySet::const_iterator it = other.m_entities.begin();
       it != other.m_entities.end();
       it++) {
    ISketchUpEntity *pEnt = *it;
    if (pEnt != NULL) {
      pEnt->AddRef();
      m_entities.insert(pEnt);
    }
  }
}

inline void SketchUpEntitySet::Add(ISketchUpEntity *pObj) {
  if (m_entities.insert(pObj).second) {
    pObj->AddRef();
  }
}

inline void SketchUpEntitySet::Get(const ISketchUpEntitySetFilter *filter,
                                   ISketchUpCollection *collection) const {
  for (SkpEntitySet::const_iterator it = m_entities.begin();
       it != m_entities.end(); it++) {
    ISketchUpEntity *pEnt = *it;
    if (pEnt != NULL) {
      if (filter->Accept(pEnt)) {
        collection->Add(pEnt);
      }
    }
  }
}

inline int SketchUpEntitySet::GetCount() const {
  return (int) m_entities.size();
}

}  // end namespace sketchup
}  // end namespace atlast

#endif  // SKETCHUP_SOURCE_SKETCHUP_SKPWRITER_SAPI_DEFS_H_
