// Copyright 2010 Google Inc. All Rights Reserved.
#ifndef SKETCHUP_SOURCE_SKETCHUP_SKPWRITER_SAPI_ICONTROLLER_H_
#define SKETCHUP_SOURCE_SKETCHUP_SKPWRITER_SAPI_ICONTROLLER_H_

#if _MSC_VER > 1000
#pragma once
#endif  // _MSC_VER > 1000

#include "source/sketchup/skpwriter/sapi/ientity.h"
namespace atlast {
namespace sketchup {
class ISketchUpEdge;
class ISketchUpDrawingElement;

/**
 * @interface ISketchUpController
 * @brief A controller object for actions that involve more than
 * the basic properties of a single object.
 */
class ISketchUpController :
  public ISketchUpEntity {
public:
  /**
   * @brief every object knows whether it is valid.
   * @return true iff this is a valid object.
   */
  virtual bool IsValid() const = 0;
  /**
   * @brief Transforms all the objects in the given set.
   * @param eSet A collection containing DrawingElements and Vertices
   * to be transformed. It is assumed this is a "set" in the sense that
   * entities are not duplicated.
   * @param trans The transformation to use.
   * @return true iff the transform worked.
   */
  virtual bool Transform(const ISketchUpGeometricElementVector *eSet,
                         const atlast::geometry::CTransformation &trans) = 0;

  /** 
   * @brief Transforms the given element along with connected
   * objects.  For example, transforming a face that has a
   * component attached to it will result in the transforming
   * the attached component.
   * @param pElement The object to transform.
   * @param trans The transformation to use.
   * @return true iff the transform worked.
   */
  virtual bool Transform(ISketchUpGeometricElement *pElement,
                         const atlast::geometry::CTransformation &trans) = 0;

  /** 
   * @brief Transforms the entities in the given container.
   * @param pContainer The container to transform.
   * @param trans The transformation to use.
   * @return true iff the transform worked.
   */
  virtual bool Transform(ISketchUpEntityContainer *pContainer,
                         const atlast::geometry::CTransformation &trans) = 0;
  /**
   * @brief Erase the elements in the given set. 
   * 
   * Any dependent
   * objects will also be erased. For example, if an edge is 
   * erased, the faces attached to the edge will also be erased.
   * @param eSet The set of objects to erase.
   */
  virtual void Erase(const ISketchUpDrawingElementVector *eSet) = 0;
  /**
   * @brief Erase the elements in the given array. 
   * 
   * Any dependent
   * objects will also be erased. For example, if an edge is 
   * erased, the faces attached to the edge will also be erased.
   * \param objs The set of objects to erase.
   * \param nObjs The number of objects in the array
   */
  virtual void Erase(ISketchUpDrawingElement **objs, int nObjs) = 0;
  /**
   * @brief Erase the given element. Any dependent
   * objects will also be erased. For example, if an edge is 
   * erased, the faces attached to the edge will also be erased.
   * @param pElement The object to erase.
   */
  virtual void Erase(ISketchUpDrawingElement *pElement) = 0;

  /**
   * @brief Erase the elements in the given selection set. Any dependent
   * objects will also be erased. For example, if an edge is 
   * erased, the faces attached to the edge will also be erased.
   * @param pSS The selection set of objects to erase.
   */
  virtual void Erase(ISketchUpSelectionSet *pSS) = 0;
  /**
   * @brief Retrieves all the elements connected to the given element.
   *
   * While the return value is a vector, it obeys set semantics in that
   * each element is represented exactly once.
   *
   * @param pElement The element to seed the traversal.
   * @return The set of all connected elements.  This will include
   * Faces and Edges, not Vertices.
   */
  virtual ISketchUpDrawingElementVectorPtr
    GetAllConnected(ISketchUpDrawingElement *pElement) = 0;

  /**
   * @brief Extrude the given face in the direction of its normal
   * the amount given in the <code>distance</code> parameter.
   * @param pFace The face to extrude.
   * @param distance The amount to extrude.  Can be positive or negative.
   */
  virtual void Extrude(ISketchUpFace *pFace, double distance) = 0;

  /**
   * \brief Merge all the coplanar faces in the given container.
   * \param pContainer The container.
   * \param bRecurse If true, will recurse through all used components.
   */
  virtual void MergeCoplanarFaces(ISketchUpEntityContainer *pContainer,
                                  bool bRecurse) = 0;

  /**
   * \brief Fixes any errors found in the given container.
   * \param pContainer The container.
   */
  virtual void FixErrors(ISketchUpEntityContainer *pContainer) = 0;

  /**
   * \brief Fixes any errors found in the given model.
   * \param pModel The model.
   */
  virtual void FixErrors(ISketchUpModel *pModel) = 0;
};
}
}
#endif  // SKETCHUP_SOURCE_SKETCHUP_SKPWRITER_SAPI_ICONTROLLER_H_
