// Copyright 2010 Google Inc. All Rights Reserved.
#ifndef SKETCHUP_SOURCE_SKETCHUP_SKPWRITER_SAPI_IDOCUMENT_H_
#define SKETCHUP_SOURCE_SKETCHUP_SKPWRITER_SAPI_IDOCUMENT_H_

#pragma once

#include "source/sketchup/skpwriter/sapi/defs.h"
#include "source/sketchup/skpwriter/sapi/iobject.h"

namespace atlast {
namespace sketchup {

class ISketchUpModel;
class ISketchUpSelectionSet;
class ISketchUpView;
class ISketchUpController;
class ISketchUpAngleFormatter;
class ISketchUpLengthFormatter;
/**
 * @enum SketchUpFileVersion
 * Constants defining the available file versions.
 */
typedef enum {
  MostRecent = -1,
  Version3_0 = 3,
  Version4_0 = 4,
  Version5_0 = 5,
  Version6_0 = 6,
  Version7_0 = 7
} SketchUpFileVersion;

/**
 * @class ISketchUpDocument
 * @brief The document class.  A document controls
 * which UI is active for this particular model, which
 * views are available, and undo transaction handling.
 */
class ISketchUpDocument :
  public ISketchUpObject {
public:
  /**
   * @brief Save the current document to a file at the given filename.
   * @param fileName The name of the file.
   * \param version The SketchUp file version.
   */
  virtual void SaveFile(LPCWSTR fileName, SketchUpFileVersion version =
                        MostRecent) THROWS(ISketchUpException&) = 0;

  virtual bool OpenFile(LPCWSTR fileName) = 0;
  /**
   * @brief Get the Model held by this Document.
   *
   * @return The model held by this document.
   */
  virtual ISketchUpModelPtr GetModel() const = 0;

  /**
   * @brief Retrieves the current selection set.
   *
   * @return The current selection set.
   */
  virtual ISketchUpSelectionSetPtr GetCurrentSelectionSet() const = 0;

  /**
   * @brief Selects the given tool to be active.  
   * Note that this will do nothing if the current application does
   * not support tools.
   * @param pTool The tool to make active.
   */
  virtual void SelectTool(sketchup::ISketchUpTool *pTool) = 0;

  /**
   * @brief Retrieves the active view.
   * @return The active View.  Note that this may be NULL if the
   * current application doesn't support views.
   */
  virtual ISketchUpViewPtr GetActiveView() const = 0;


  /**
   * @brief Writes text to the status bar of the application.
   *
   * This method is a no-op in the case that the application does not have
   * a status bar.
   * @param text The text to write.
   * @param paneNumber The number of the pane.
   */
  virtual void SetStatusText(LPCWSTR text, unsigned int paneNumber = 0) = 0;

  /**
   * @brief Retrieves the controller associated with the current
   * document.
   * @return The controller object.
   */
  virtual ISketchUpControllerPtr GetController() const = 0;

  /**
   * @brief Enable or disable Undo handling.
   * 
   * See the Undo management discussion in \ref UNDO.
   * @param disable If true, undo handling is disabled. Otherwise it
   * is enabled.
   */
  virtual void DisableUndo(bool disable) = 0;

  /**
   * @brief Start an undo transation with the given name.
   * See the Undo management discussion in \ref UNDO.
   * @param transactionName The name of the transaction.
   */
  virtual void StartTransaction(LPCWSTR transactionName) = 0;

  /**
   * @brief Commit the current transaction.
   * See the Undo management discussion in \ref UNDO.
   */
  virtual void CommitTransaction() = 0;

  /**
   * @brief Abort the current transaction.
   * See the Undo management discussion in \ref UNDO.
   */
  virtual void AbortTransaction() = 0;

  /**
   * @brief Get the Length Formatter
   *
   * @return The Length formatter using this document's options.
   */
  virtual ISketchUpLengthFormatterPtr GetLengthFormatter() const = 0;
  /**
   * @brief Get the Angle Formatter
   * @return The Angle formatter using this document's options.
   */
  virtual ISketchUpAngleFormatterPtr GetAngleFormatter() const = 0;
protected:
  virtual ~ISketchUpDocument() {}
};
}
}
#endif  // SKETCHUP_SOURCE_SKETCHUP_SKPWRITER_SAPI_IDOCUMENT_H_
