// Copyright 2010 Google Inc. All Rights Reserved.
#ifndef SKETCHUP_SOURCE_SKETCHUP_SKPWRITER_SAPI_IEDGEUSE_H_
#define SKETCHUP_SOURCE_SKETCHUP_SKPWRITER_SAPI_IEDGEUSE_H_

#if _MSC_VER > 1000
#pragma once
#endif  // _MSC_VER > 1000

#include "source/sketchup/skpwriter/sapi/ientity.h"

namespace atlast {
namespace sketchup {
class ISketchUpFace;
class ISketchUpLoop;
class ISketchUpEdge;

/**
 * @interface ISketchUpEdgeUse
 * @brief An EdgeUse interface.
 *
 * An edge use is used by loops to identify which edges bound
 * the loop.  Since the loop requires counter-clockwise traversal
 * about its normal, some of the edges that make up the loop may
 * be traversed in reverse order (as compared to their natural
 * direction). The EdgeUse captures this information.
 *
 * EdgeUses are linked in a circular list, which can be traversed
 * via the Previous and Next methods.
 *
 * In addition, the edge uses maintain a link to the other edge uses
 * used by the same face.
 * @see ISketchUpFace
 * @see ISketchUpEdge
 * @see ISketchUpLoop
 */
class ISketchUpEdgeUse :
  public ISketchUpEntity {
public:
  /**
   * @brief Retrieves the corresponding Edge.
   *
   * As an EdgeUse is essentially a reference to an Edge,
   * this method retrieves the underlying edge object.
   *
   * If this object is not valid, NULL is returned.
   * @return The underlying Edge.
   */
  virtual ISketchUpEdgePtr GetEdge() const = 0;

  /**
   * \brief Retrieves the face which is partially defined by this
   * edge use.
   *
   * If this object is not valid, NULL is returned.
   *
   * @return The face partially bounded by this EdgeUse.
   */
  virtual ISketchUpFacePtr GetFace() const = 0;

  /**
   * @brief Gets the loop in which this EdgeUse is a participant.
   *
   * If this object is not valid, NULL is returned.
   *
   * @return The loop containing this EdgeUse.
   */
  virtual ISketchUpLoopPtr GetLoop() const = 0;

  /**
   * @brief Determines whether this EdgeUse is reversed
   * relative to the underlying edge.
   *
   * If this object is not valid, false is returned.
   *
   * @return true iff this EdgeUse is reversed relative to its
   * underlying Edge.
   */
  virtual bool IsReversed() const = 0;

  /**
   * @brief Gets the next EdgeUse in this loop.
   *
   * If this object is not valid, NULL is returned.
   * 
   * @return The next edge use in the loop.
   */
  virtual ISketchUpEdgeUsePtr GetNext() const = 0;
  /**
   * @brief Gets the previous EdgeUse in this loop.
   *
   * If this object is not valid, NULL is returned.
   * 
   * @return The previous edge use in the loop.
   */
  virtual ISketchUpEdgeUsePtr GetPrevious() const = 0;
};
}
}
#endif  // SKETCHUP_SOURCE_SKETCHUP_SKPWRITER_SAPI_IEDGEUSE_H_
