// Copyright 2010 Google Inc. All Rights Reserved.
#ifndef SKETCHUP_SOURCE_SKETCHUP_SKPWRITER_SAPI_IENTITYCONTAINER_H_
#define SKETCHUP_SOURCE_SKETCHUP_SKPWRITER_SAPI_IENTITYCONTAINER_H_

#if _MSC_VER > 1000
#pragma once
#endif  // _MSC_VER > 1000

#include <vector>
#include <utility>
#include "source/sketchup/skpwriter/sapi/ientity.h"
#include "source/common/geometry/point3d.h"
#include "source/common/geometry/vector3d.h"
#include "source/common/geometry/polygonmesh.h"

#include "source/sketchup/skpwriter/sapi/icurve.h"
#include "source/sketchup/skpwriter/sapi/ivertex.h"
#include "source/sketchup/skpwriter/sapi/iedge.h"
#include "source/sketchup/skpwriter/sapi/iface.h"

namespace atlast {
namespace sketchup {

class ISketchUpEdge;
class ISketchUpComponentInstance;
class ISketchUpComponentDefinition;
class ISketchUpVertex;
class ISketchUpFace;
class ISketchUpPolyline;
class ISketchUpArc;
class ISketchUpCurve;
class ISketchUpLineSegment;
class ISketchUpAttributeDictionary;

#define HIDE_BASED_ON_INDEX     1
#define SOFTEN_BASED_ON_INDEX   2
#define AUTO_SOFTEN             4
#define SMOOTH_SOFT_EDGES       8

/**
 * \class EdgeDescriptor
 * \brief Describes an Edge.
 */
struct EdgeDescriptor {
  EdgeDescriptor()
    : soft(false)
      ,
      smooth(false)
      ,
      hidden(false)
      ,
      pLayer(NULL) {}

  /** Is this edge soft? */
  bool soft;
  /** Is this edge smooth? */
  bool smooth;

  /** Is this edge hidden? */
  bool hidden;

  /** The Layer this edge lives on. */
  ISketchUpLayer *pLayer;
};

class EdgeDescriptors {
public:
  virtual ~EdgeDescriptors() {}
  virtual EdgeDescriptor GetDescriptor(int i) const = 0;
  virtual int GetCount() const = 0;
};

/**
 * \class EdgeDescriptorVector
 * \brief Utility implementation of the EdgeDescriptor interface
 */
class EdgeDescriptorVector : public EdgeDescriptors {
public:
  EdgeDescriptorVector(int size)
    : m_size(size) {
    m_descriptors = new EdgeDescriptor[size];
  }

  EdgeDescriptorVector(const EdgeDescriptors &descs) {
    m_descriptors = new EdgeDescriptor[descs.GetCount()];
    for (int i = 0; i < descs.GetCount(); i++) {
      m_descriptors[i] = descs.GetDescriptor(i);
    }
  }

  virtual ~EdgeDescriptorVector() {
    delete [] m_descriptors;
  }

  EdgeDescriptorVector &operator=(const EdgeDescriptors &descs) {
    delete [] m_descriptors;
    m_descriptors = new EdgeDescriptor[descs.GetCount()];
    for (int i = 0; i < descs.GetCount(); i++) {
      m_descriptors[i] = descs.GetDescriptor(i);
    }

    return *this;
  }

  void Insert(int i, const EdgeDescriptor &descriptor) {
    ASSERT(i >= 0 && i < m_size);
    m_descriptors[i] = descriptor;
  }

  virtual int GetCount() const {
    return m_size;
  }

  virtual EdgeDescriptor GetDescriptor(int i) const {
    ASSERT(i >= 0 && i < m_size);
    if (i >= 0 && i < m_size) {
      return m_descriptors[i];
    }
    EdgeDescriptor empty;
    return empty;
  }

  int m_size;
  EdgeDescriptor *m_descriptors;
};

/**
 * \class MaterialDescriptor
 * \brief Describes the application of a material to a particular face.
 */
struct MaterialDescriptor {
  /** 
   * \brief Empty constructor. 
   * Initializes:
   * - pMaterial = NULL
   * - nUVs = 0
   * - pUVs = NULL
   * - pXYZIndices = NULL
   */
  MaterialDescriptor()
    : pMaterial(NULL)
      ,
      nUVs(0)
      ,
      pUVs(NULL)
      ,
      pXYZIndices(NULL),
      num_pins(0) {
        memset(pushpins, 0, 16 * sizeof(double));
        memset(projection, 0, 3 * sizeof(double));
      }

  /** Default destructor: does nothing. */
  virtual ~MaterialDescriptor() {}

  /** The material */
  ISketchUpMaterialPtr pMaterial;

  /** Can have 0 to 4 points. */
  int nUVs;

  /** Space for the UV points.  Must contain at least nUVs items.*/
  atlast::geometry::CPoint2d *pUVs;

  /** Space for the XYZ (model space) points.  Must contain at least nUVs items.*/
  int *pXYZIndices;

  /** Texture face info pushpins.  There are up to four pushins.
      Each pushpin has 2 pairs of doubles, a 
      uv pair for the pushpin texture coordinate, and an xy pair for the 
      arbitrary axis point which is the origin of Cartesian coordinate 
      system of the uv pair.  The pushpins values are laid out in the following
      manner:
      pushpin[0] = u0; pushpin[1] = v0;
      pushpin[2] = x0; pushpin[3] = y0;
      pushpin[4] = u1; pushpin[5] = v1;
      pushpin[6] = x1; pushpin[7] = y1;
      pushpin[8] = u2; pushpin[9] = v2;
      pushpin[10] = x2; pushpin[11] = y2;
      pushpin[12] = u3; pushpin[13] = v3;
      pushpin[14] = x3; pushpin[15] = y3;
      */
  double pushpins[16];
  unsigned num_pins;
  /** The projection unit vector, if the texture is projected along a vector.
      Value of (0, 0, 0) indicates a projection vector is not used. */
  double projection[3];
};

/**
 * \class FaceLoop
 * \brief Structure holding the definition of a loop inside a face.
 * 
 * This struct is used to describe inner loops (holes) in a face.
 * 
 * This structure is used in the ISketchUpEntityContainer::Fill method, and depends on the arguments
 * therein. 
 */
struct FaceLoop {
  /** Initializes data to zero or NULL. */
  FaceLoop()
    : nIndices(0)
      ,
      pIndices(NULL)
      ,
      pEdgeDescriptors(NULL) {}

  /** Destructor does nothing. */
  virtual ~FaceLoop() {}

  /** The number of points in the loop */
  int nIndices;

  /** The points defining the loop */
  int *pIndices;

  /** EdgeDescriptors - can be NULL */
  EdgeDescriptors *pEdgeDescriptors;
};

/**
 * \class FaceDescriptor
 * \brief Describes a Face.
 *
 * This structure is used in the ISketchUpEntityContainer::Fill method, and depends on the arguments
 * therein. 
 * 
 * A face is defined by several things:
 * - Its outer loop
 * - Whether it should be reversed
 * - Its (possibly empty) collection of inner loops
 * - Optional front and back materials
 */
struct FaceDescriptor {
  /**
   * \brief Constructor.
   *
   * This constructor initializes:
   * - pFaceLoop = NULL
   * - pFrontMaterialDescriptor = NULL
   * - pBackMaterialDescriptor = NULL
   * - bReverse = false
   * - nInnerLoops = 0
   * - pInnerLoops = NULL
   * - pLayer = NULL
   */
  FaceDescriptor()
    : pFaceLoop(NULL)
      ,
      pFrontMaterialDescriptor(NULL)
      ,
      pBackMaterialDescriptor(NULL)
      ,
      pLayer(NULL)
      ,
      bReverse(false)
      ,
      pInnerLoops(NULL)
      ,
      nInnerLoops(0) {}

  /** Destructor does nothing. */
  virtual ~FaceDescriptor() {}

  /** The outer loop defining the face. */
  FaceLoop *pFaceLoop;
  /** Material description for the front face. */
  MaterialDescriptor *pFrontMaterialDescriptor;
  /** Material description for the back face. */
  MaterialDescriptor *pBackMaterialDescriptor;

  /** The layer this face should live on. */
  ISketchUpLayer *pLayer;

  /** This parameter is true iff we should reverse the face. */
  bool bReverse;

  /** 
   * A (possibly NULL) pointer to an array of FaceLoop objects defining the inner
   * loops in this face.
   */
  FaceLoop *pInnerLoops;
  /**
   * The number of inner loops provided.  By default, it's 0.
   */
  int nInnerLoops;
};


/**
 * @interface ISketchUpEntityContainer
 * @brief An interface covering those objects that can contain and
 * create other Entities.
 * 
 * This class can be used as a factory, with all the create methods
 * contained within.  Each create method will create the desired object
 * and add it to "this" object.
 *
 * This class can also be used to query for objects of the underlying
 * types.  It is a container in that sense.
 */
class ISketchUpEntityContainer :
  public ISketchUpEntity {
public:

  /**
   * \brief Fills an empty EntityContainer with faces.
   *
   * This method is the fastest way to populate an EntityContainer with faces. 
   * This method achieves this performance by making certain assumptions about the data,
   * and the state of the container:
   * - The container must be empty.
   * - The data must be consistent: no duplicate faces or points, inner loops live within the 
   * given faces, etc.
   *
   * By virtue of the FaceDescriptor, EdgeDescriptor and MaterialDescriptor a client can 
   * create textured faces with zero or more inner loops.  The user also has control over the 
   * basic attributes of each edge: hidden, soft and smooth.  Again, it is the responsibility
   * of the user to ensure that the data is consistent.
   *
   * \param pnts An array of all the vertices used by the faces and the inner loops.
   * \param numPoints The number of points used by the outer loops of the faces.  This number is
   * less than or equal to the total number of points in the pnts parameter.
   * \param faces An array of FaceDescriptor objects.
   * \param numFaces The number of items in the faces parameter.
   * \param weldVertices If true, all coincident vertices will be joined.  The default is false since 
   * this is a slight performance hit.
   * \return true iff all the faces were created.
   */
  virtual bool Fill(atlast::geometry::CPoint3d pnts[], int numPoints,
                    FaceDescriptor faces[], int numFaces, bool weldVertices =
                    false) = 0;

  virtual bool CreateTerrain(atlast::geometry::CPoint3d pnts[], int numPoints,
                             bool weldVertices, FaceDescriptor faces[],
                             int numFaces, ISketchUpMaterial *pMaterial,
                             atlast::geometry::CPoint3d bnds[4]) = 0;
  /**
   * \brief Creates an Edge in this container.
   *
   * The two given vertices must be valid for this to succeed.  
   * \throw ISketchUpException In the case of invalid input - in particular,
   * implementations of the ISketchUpVertex interface that are not recognized
   * internally.
   * \param pStart The start vertex for the edge.
   * \param pEnd The end vertex for the edge.
   * \return The edge between the start and end vertices.
   * \todo Are there restrictions on the "parent" of the vertices?
   */
  virtual ISketchUpEdgePtr CreateEdge(ISketchUpVertex *pStart,
                                      ISketchUpVertex *pEnd) THROWS(ISketchUpException&)
    = 0;


  virtual ISketchUpComponentInstancePtr
    CreateComponentInstance(const ISketchUpComponentDefinition
                            *pComponentDefinition)
    THROWS(ISketchUpException&) = 0;

  virtual ISketchUpFacePtr CreateFace(const ISketchUpCurveVector *outerLoop)
    THROWS(ISketchUpException&) = 0;

  virtual ISketchUpFacePtr CreateFace(atlast::geometry::CPoint3d points[],
                                      int numPoints) = 0;

  virtual ISketchUpFacePtr CreateFace(const ISketchUpVertexVector *verts) = 0;

  /**
   * \brief Create a collection of faces from a polygon mesh.
   * \param mesh The mesh used to create the faces.
   * \param flags A combination of 
   * <ul>
   * <li> HIDE_BASED_ON_INDEX
   * <li> SOFTEN_BASED_ON_INDEX
   * <li> AUTO_SOFTEN
   * <li> SMOOTH_SOFT_EDGES
   * </ul>
   * The default value is <code>AUTO_SOFTEN | SMOOTH_SOFT_EDGES</code>.
   * \param pFrontMaterial The material to use on the front sides of the faces.
   * \param pBackMaterial The material to use on the back sides of the faces.
   * \param ppFaces If not NULL, filled in with the created faces.
   * \return The number of faces created.
   */
  virtual long CreateFaces(const atlast::geometry::CPolygonMesh &mesh,
                           unsigned int flags = 0,
                           ISketchUpMaterial *pFrontMaterial = NULL,
                           ISketchUpMaterial *pBackMaterial = NULL,
                           ISketchUpFaceVectorPtr *ppFaces = NULL) = 0;

  virtual ISketchUpArcPtr CreateArc(const atlast::geometry::CPoint3d &center,
                                    const atlast::geometry::CPoint3d
                                    &startPoint,
                                    const atlast::geometry::CPoint3d &endPoint,
                                    const atlast::geometry::CUnitVector3d
                                    &normal, int numSegs = 12) = 0;

  virtual ISketchUpPolylinePtr CreatePolyline(const ISketchUpLineSegmentVector
                                              *) = 0;

  virtual ISketchUpLineSegmentPtr CreateLineSegment(ISketchUpVertex *pStart,
                                                    ISketchUpVertex *pEnd) = 0;
  virtual ISketchUpLineSegmentPtr CreateLineSegment(const atlast::geometry::
                                                    CPoint3d &start,
                                                    const atlast::geometry::
                                                    CPoint3d &end) = 0;

  virtual ISketchUpGroupPtr CreateGroup() = 0;

  virtual ISketchUpEntityVectorPtr GetEntities() = 0;

  virtual ISketchUpVertexVectorPtr GetVertices() = 0;

  virtual ISketchUpEdgeVectorPtr GetEdges() =0;
  virtual ISketchUpFaceVectorPtr GetFaces() =0;

  virtual atlast::geometry::CBoundingBox3d GetBoundingBox() const = 0;
};
}
}
#endif  // SKETCHUP_SOURCE_SKETCHUP_SKPWRITER_SAPI_IENTITYCONTAINER_H_
