// Copyright 2010 Google Inc. All Rights Reserved.
#ifndef SKETCHUP_SOURCE_SKETCHUP_SKPWRITER_SAPI_IFACE_H_
#define SKETCHUP_SOURCE_SKETCHUP_SKPWRITER_SAPI_IFACE_H_

#if _MSC_VER > 1000
#pragma once
#endif  // _MSC_VER > 1000
#include "source/sketchup/skpwriter/sapi/ientity.h"
#include "source/sketchup/skpwriter/sapi/ivertex.h"
#include "source/sketchup/skpwriter/sapi/iedge.h"
#include "source/sketchup/skpwriter/sapi/icurve.h"
#include "source/sketchup/skpwriter/sapi/iloop.h"
#include "source/common/geometry/polygonmesh.h"  // AtLast
namespace atlast {
namespace sketchup {

/**
* @interface ISketchUpFace
* @brief A Face interface.
*
* An ISketchUpFace is defined by an outer loop and a collection of inner loops.
* The outer loop is always traversed counter clockwise with respect to the
* normal of the face.
*
* A loop is a connected collection of ISketchUpEdgeUse objects.  An ISketchUpEdgeUse references an
* ISketchUpEdge along with a direction: is the ISketchUpEdgeUse using the ISketchUpEdge in the same
* direction as the Edge's "natural" orientation?
*
* To better understand this, consider the following situation where one finds two
* polygons \a P1 and \a P2:
*
* \image html PolygonsWithEdges.PNG
*
* The edges are oriented in such a way that \a P1's edges are traversed counterclockwise
* about its normal (assuming the normal extends out of the screen, as opposed to into
* the screen), whereas \a P2's edges are traversed clockwise.
*
* In SketchUp, a face is defined by an outer loop which is traversed counterclockwise
* with respect to the face's normal.  This loop object must somehow be defined in terms
* of the edges in the system, but clearly in this example there is a problem with using
* the edges directly for \a P2.
*
* The SketchUp modeler addresses this problem by introducing edgeuses, which are
* little more than an object that encapsulates a reference to an edge along with a
* flag indicating whether the edgeuse should be traversed in the same direction as the edge,
* or reversed.
*
* If we draw in the edge uses (denoted \a e#) then we might have:
*
* \image html PolygonsWithEdgeUses.PNG
*
* The polygon denoted \a P1 is enclosed by the edges \a E1, \a E2, \a E3 and \a E4 via reference through edgeuses
* \a e1, \a e2, \a e3 and \a e4.    These edgeuses do not reverse their orientation via their respective edges.
*
* Openings are cuts in the face occuring because of a cut-component being
* attached to the face.

*
* A Face is created via a ISketchUpEntityContainer.
*
* @see ISketchUpEntityContainer
* @see ISketchUpEdge
* @see ISketchUpLoop
* @see ISketchUpVertex
* @see ISketchUpMaterial
*/

class ISketchUpFace : public ISketchUpDrawingElement {
public:
  /**
   * \brief Adds an inner loop to the face.
   *
   * The inner loop effectively defines a "hole" in the face.
   * The inner loop must truly be <i>inner</i> - it must lie within
   * the face's outer loop.
   * \throw ISketchUpException Unrecognized input types - typically ISketchUpCurvePtr
   * implementations that are foreign to this implementation.
   * \param curves The curves defining the inner loop.
   */
  virtual void AddInnerLoop(const ISketchUpCurveVector *curves)
    THROWS(ISketchUpException&) = 0;

  virtual void AddInnerLoop(const atlast::geometry::CPoint3d vertices[],
                            int numPoints) = 0;

  /**
   * \brief Retrieves the outer loop of this face.
   *
   * If this face is not valid, NULL is returned.
   *
   * \return The outerloop.
   */
  virtual ISketchUpLoopPtr GetOuterLoop() const = 0;

  /**
   * \brief Returns a vector of the inner loops for this
   * face.
   * 
   * If this face is not valid, an empty vector is returned.
   *
   * \return A vector of the inner loops for this face.
   */
  virtual ISketchUpLoopVectorPtr GetInnerLoops() const = 0;

  /**
   * \brief Directs the face to reverse its orientation.
   * 
   * Effectively reverses the normal of the face's plane.
   *
   * \param reverseIt If true, the face is reversed.  If
   * false, the method is a no-op.
   */
  virtual void SetReversed(bool reverseIt = true) = 0;

  virtual ISketchUpMaterialPtr GetFrontMaterial() = 0;
  virtual void SetFrontMaterial(const ISketchUpMaterial *pMaterial) = 0;
  virtual void SetFrontMaterial(const ISketchUpMaterial *pMaterial,
                                const ISketchUpVertexVector *vertices,
                                atlast::geometry::CPoint2d uvCoords[])
    THROWS(ISketchUpException&)= 0;
  virtual void SetFrontMaterial(const ISketchUpMaterial *pMaterial,
                                atlast::geometry::CPoint3d vertices[],
                                int numPoints,
                                atlast::geometry::CPoint2d uvCoords[])
    THROWS(ISketchUpException&)= 0;

  virtual ISketchUpMaterialPtr GetBackMaterial() = 0;
  virtual void SetBackMaterial(const ISketchUpMaterial *pMaterial) = 0;
  virtual void SetBackMaterial(const ISketchUpMaterial *pMaterial,
                               const ISketchUpVertexVector *vertices,
                               atlast::geometry::CPoint2d uvCoords[])
    THROWS(ISketchUpException&)= 0;
  virtual void SetBackMaterial(const ISketchUpMaterial *pMaterial,
                               atlast::geometry::CPoint3d vertices[],
                               int numPoints,
                               atlast::geometry::CPoint2d uvCoords[])
    THROWS(ISketchUpException&)= 0;

  virtual void GetNormal(double *x, double *y, double *z) const = 0;
  virtual atlast::geometry::CUnitVector3d GetNormal() const = 0;

  virtual double GetArea() const = 0;
  /**
   * \brief Retrieves the plane defined by the face.
   * 
   * If the face is not valid, a default plane is returned.
   * 
   * \return The plane defined by the points on the face and the
   * face's orientation.
   */
  virtual atlast::geometry::CPlane GetPlane() const = 0;

  /**
   * \brief Triangulates the face without any UV values.
   */
  virtual atlast::geometry::CPolygonMesh GetTriangulation() const = 0;

  /**
  * \brief Assigns the pushpin values to texture coordinates of the 
  * front side of the face.
  * \param length The number of pushpins to assign.  Valid values range from
  * 0 to 4.
  * \param uvCoords The uv texture coordinates of each pushpin.
  * \param aaCoords The arbitrary axis point of each pushpin.
  * \param projection_vector The projection vector of the texture.  A 
  * projection vector of 0.0, 0.0, 0.0 indicates that a projection 
  * vector is not used.
  * If this face is not valid, an empty vector is returned.
  *
  */
  virtual void SetFrontPushPins(const unsigned int length, 
    const atlast::geometry::CPoint2d uvCoords[],
    const atlast::geometry::CPoint2d aaCoords[],
    const atlast::geometry::CUnitVector3d& projectionVector) = 0;

  /**
  * \brief Assigns the pushpin values to texture coordinates of the 
  * back side of the face.
  * \param length The number of pushpins to assign.  Valid values range from
  * 0 to 4.
  * \param uvCoords The uv texture coordinates of each pushpin.
  * \param aaCoords The arbitrary axis point of each pushpin.
  * \param projection_vector The projection vector of the texture.  A 
  * projection vector of 0.0, 0.0, 0.0 indicates that a projection 
  * vector is not used.
  */
  virtual void SetBackPushPins(const unsigned int length, 
    const atlast::geometry::CPoint2d uvCoords[],
    const atlast::geometry::CPoint2d aaCoords[],
    const atlast::geometry::CUnitVector3d& projectionVector) = 0;
};
}
}
#endif  // SKETCHUP_SOURCE_SKETCHUP_SKPWRITER_SAPI_IFACE_H_
