// Copyright 2010 Google Inc. All Rights Reserved.
#ifndef SKETCHUP_SOURCE_SKETCHUP_SKPWRITER_SAPI_IPAGES_H_
#define SKETCHUP_SOURCE_SKETCHUP_SKPWRITER_SAPI_IPAGES_H_

#if _MSC_VER > 1000
#pragma once
#endif  // _MSC_VER > 1000

#include "source/sketchup/skpwriter/sapi/ientity.h"

namespace atlast {
namespace sketchup {

/**
 * @interface ISketchUpPage
 * @brief A <i>Page</i> is a named view in SketchUp.
 *
 */
class ISketchUpPages : public ISketchUpEntity {
 public:
  /**
   * Get the number of pages.
   */
  virtual int GetCount() const = 0;

  /**
   * Add a new page.
   * If nIndex = -1, then add it at the end.
   *
   * \return The newly added page
   */
  virtual ISketchUpPagePtr AddPage(LPCWSTR pageName, int *pResultingIndex =
                                   NULL, int nIndex = -1) = 0;

  /**
   * Remove an existing page.
   *
   * This does not delete the existing page.
   * Returns NULL if the page is not found in the document.
   */
  virtual ISketchUpPagePtr RemovePage(ISketchUpPage *pPage) = 0;

  /**
   * Find a page at a given index.
   */
  virtual ISketchUpPagePtr GetPage(int nIndex) const = 0;

  /**
   * Find a page with a given name.
   */
  virtual ISketchUpPagePtr FindPage(LPCWSTR lpszPageName) const = 0;

  /**
   * Get the zero based index of a page
   * Returns -1 if the page is not found in the model.
   */
  virtual int GetPageIndex(const ISketchUpPage *pPage) const = 0;
  /**
   * Set the zero based index of a page
   * Returns -1 if the page is not found in the model.
   */
  virtual int SetPageIndex(ISketchUpPage *pPage, int nIndex) = 0;

  /** Access to the selected page */
  virtual ISketchUpPagePtr GetSelectedPage() const = 0;

  /** Set the selected page */
  virtual void SetSelectedPage(ISketchUpPage *pPage) = 0;

  typedef enum {
    ModelDefault,
    ImageEmbedded,
    ImageEmbeddedAndLinked,
    ImageLinked
  } ImageReferenceType;

  typedef enum {
    UnitsPixels,
    UnitsNormalizedX,
    UnitsNormalizedY
  } MatchPhotoUnits;

  /** 
   * \brief Add a new page for photomatching.
   *
   * \param imageName This must be a valid image file.
   * \param camera If the camera is NULL we use the default camera, but a
   * camera will normally be provided.
   * \param pageName If the page name is NULL (or blank) we derive a default
   * unique name from the image file name.
   *
   * The following arguments are currently private, but since this API isn't
   * exposed to the public and will hopefully go away soon, I've left them
   * on this method.
   *
  * \param type
  * The image reference type will normally be ModelDefault, but it may
  * explicitly specify ImageEmbedded, ImageLinkedAndEmbedded, or
  * ImageLinked.  At present the UI for this is hidden and does not
  * expose ImageEmbedded, but the API does recognize it.
  * \param principal_poitn_delta_x,principal_point_delta_y
  * The next two arguments are delta x/y from the center of the image to
  * the principal point and can be used to supply a cropped photo.  The
  * deltas are measured from the center of the image, so 0,0 will specify
  * an uncropped photo.  Positive values are to the right and to the top of
  * the image.
  * \param focal_length
  * The "focal_length" argument can be defaulted to 0 in which case it will
  * use the "field of view" specified by the camera.  However, we often
  * know the focal length specified for an image and this provides
  * a way to override the camera fov.  The latter is tedious to calculate
  * for a cropped photo while the focal length is often known regardless of
  * how an image is cropped.
  * \param units
  * The "units" argument defines the units for the principal point and
  * focal length.  The normalized versions are useful if ppx, ppy, and f
  * are given relative to some image coordinate system and you don't happen
  * to know the size of the image (as with Escher files).
  */
  virtual ISketchUpPagePtr AddMatchPhotoPage(LPCWSTR imageName,
      CCameraDefinition *camera = NULL,
      LPCWSTR pageName = NULL,
      ImageReferenceType type = ModelDefault,
      double principal_point_delta_x = 0,
      double principal_point_delta_y = 0,
      double focal_length = 0,
      MatchPhotoUnits units = UnitsPixels) = 0;
};
}
}

#endif  // SKETCHUP_SOURCE_SKETCHUP_SKPWRITER_SAPI_IPAGES_H_
