// Copyright 2010 Google Inc. All Rights Reserved.

// Camera.h: interface for the CCameraDefinition class.

#ifndef SKETCHUP_SOURCE_SKETCHUP_UTILS_CAMERADEFINITION_H_
#define SKETCHUP_SOURCE_SKETCHUP_UTILS_CAMERADEFINITION_H_

#include "source/common/geometry/point3d.h"
#include "source/common/geometry/unitvector3d.h"

namespace atlast {
namespace sketchup {

/**
 * \class CCameraDefinition
 * \brief A Camera definition.
 */
class CCameraDefinition {
 public:
  /**
   * \brief Empty Constructor.
   */
  CCameraDefinition();
  /**
   * \brief Simple Constructor.
   * \param eye The position, or eye, of the camera.
   * \param target The target of the camera.
   * \param up The up direction of the camera.
   */
  CCameraDefinition(const atlast::geometry::CPoint3d &eye,
                    const atlast::geometry::CPoint3d &target,
                    const atlast::geometry::CUnitVector3d &up);

  /**
   * \brief Copy constructor
   * \param camera The camera from which all parameters should be copied.
   */
  CCameraDefinition(const CCameraDefinition &camera);
  /**
   * \brief Destructor.
   */
  virtual ~CCameraDefinition();

  /**
   * \brief Checks to make sure this is a valid camera.
   */
  bool IsValidObj() const;

  /**
   * \brief Assignment operator.
   * \param camera The object to copy.
   * \retval This object.
   */
  CCameraDefinition &operator=(const CCameraDefinition &camera);

  /**
   * \brief Comparison operator.
   * \param camera The object to compare against.
   * \retval Returns true iff the two cameras are equivalent.
   */
  bool operator==(const CCameraDefinition &camera) const;

  // Set the camera from the parameters of another camera
  void Set(const CCameraDefinition &camera);

  // Set the parameters of the camera
  void Set(const atlast::geometry::CPoint3d &eye,
           const atlast::geometry::CPoint3d &target,
           const atlast::geometry::CUnitVector3d &up, double nearDist,
           double farDist, bool bPerspective, double fov, double height);

  void Set(const atlast::geometry::CPoint3d &eye,
           const atlast::geometry::CUnitVector3d &dir,
           const atlast::geometry::CUnitVector3d &up, double nearDist,
           double farDist, bool bPerspective, double fov, double height);

  // Set the orientation of the camera
  void Set(const atlast::geometry::CPoint3d &eye,
           const atlast::geometry::CPoint3d &target,
           const atlast::geometry::CUnitVector3d &up);
  void Set(const atlast::geometry::CPoint3d &eye,
           const atlast::geometry::CUnitVector3d &dir,
           const atlast::geometry::CUnitVector3d &up);


  const atlast::geometry::CPoint3d &GetEye() const {
    return m_eye;
  }
  const atlast::geometry::CPoint3d &GetTarget() const {
    return m_target;
  }
  const atlast::geometry::CUnitVector3d &GetUp() const {
    return m_up;
  }

  double GetNear() const {
    return m_near;
  }
  double GetFar() const {
    return m_far;
  }

  bool GetPerspective() const {
    return m_bPerspective;
  }
  void SetPerspective(bool bPerspective);

  double GetFieldOfView() const {
    return m_fov;
  }
  void SetFieldOfView(double fov) {
    m_fov = fov;
  }
  void SetFieldOfView(double fov, bool bForHeight) {
    m_fov = fov;
    m_bFOVIsHeight = bForHeight;
  }

  // Computes the field of view in the height direction
  double GetHeightFieldOfView(double aspectRatio) const;

  double GetHeight() const {
    return m_height;
  }
  void SetHeight(double height) {
    m_height = height;
  }

  // Get the image width
  double GetImageWidth() const {
    return m_imageWidth;
  }
  void SetImageWidth(double w) {
    m_imageWidth = w;
  }

  // Compute the focal length in mm from the FOV and image width
  // These only apply to perspective cameras
  double GetFocalLength() const;
  void SetFocalLength(double fl);

  double AngleToFocalLength(double angle) const;
  double FocalLengthToAngle(double fl) const;

  atlast::geometry::CPoint3d GetCenter() const {
    return m_center;
  }
  void SetCenter(const atlast::geometry::CPoint3d &center) {
    m_center = center;
  }

  double GetCameraAspectRatio() const {
    return m_cameraAspectRatio;
  }
  void SetCameraAspectRatio(double ar) {
    m_cameraAspectRatio = ar;
  }

  void SetFieldOfViewIsHeight(bool bForHeight) {
    m_bFOVIsHeight = bForHeight;
  }
  bool GetFieldOfViewIsHeight() const {
    return m_bFOVIsHeight;
  }

  CString GetDescription() const {
    return m_description;
  }
  void SetDescription(const CString &description) {
    m_description = description;
  }

  // These methods return or set computed values
  atlast::geometry::CUnitVector3d GetDirection() const;
  double GetDistance() const;
  void SetDistance(double distance);

  // This checks the parameters and makes sure that we have a valid camera
  bool MakeValid();



private: // data

  atlast::geometry::CPoint3d m_eye;
  atlast::geometry::CPoint3d m_target;
  atlast::geometry::CUnitVector3d m_up;

  double m_near;
  double m_far;

  bool m_bPerspective;

  // This is the field of view specified by the user.  It could be measured
  // in either the horizontal or vertival directions.
  double m_fov;

  // For a parallel camera, we specify the height rather than the field of view.
  double m_height;	// used for orthographic camera

  // This is the aspect ratio that is wanted for a selected camera.
  // This value is independent of the actual window aspect ratio.
  // If this value is 0, it means don't use it.
  double m_cameraAspectRatio;

  // This controls whether the field of view is measured by the
  // height or width of the view
  bool m_bFOVIsHeight;

  // For a fixed field of view camera you cannot change the field of view.
  bool m_bFixedFOV;

  // This is not currently used.  It is reserved for future use
  // in case we want to support oblique projections
  atlast::geometry::CPoint3d m_center;

  // A description of the Camera.  Since CCameraDefinition is not derived from
  // CEntity we cannot attach attributes to it, but this allows us
  // to attach some kind of identifying information
  CString m_description;

  // This is the width of the image projected onto the "film".  It is used
  // to convert from lens angle (field of view) to focal length.
  // If this is not set, it uses 36mm for the width to agree with
  // what we were doing prior to 4.0
  double m_imageWidth;
};
}
}

#endif  // SKETCHUP_SOURCE_SKETCHUP_UTILS_CAMERADEFINITION_H_
