// Copyright 2010 Google Inc. All Rights Reserved.

#ifndef SKETCHUP_SOURCE_SKETCHUP_UTILS_IREFCOUNTOBJECT_H_
#define SKETCHUP_SOURCE_SKETCHUP_UTILS_IREFCOUNTOBJECT_H_
#ifdef _WINDOWS
#   include <atlconv.h>
#elif __APPLE__
#   include "source/sketchup/macwin/mfcbase.h"
#endif
#ifdef __LINUX__
#ifndef ASSERT
#include <assert.h>
#define ASSERT(x) assert(x)
#endif
#endif
#pragma once
#include <typeinfo>
namespace atlast {
namespace utils {
/**
 * @interface IRefCountObject
 * @brief The base of all reference counted object.
 * 
 * This is similar to the COM <code>IUnknown</code> interface, without
 * the need for a <code>QueryInterface</code> method, as we use 
 * <code>dynamic_cast</code>.  Code using this should be compiled
 * with runtime class identification turned on, if needed.
 *
 * @see RefCountObjectImpl
 */
class IRefCountObject {
public:
  virtual ~IRefCountObject() {}

  /**
   * @brief Notifies this object that another object is referencing
   * this object.
   */
  virtual unsigned long AddRef() = 0;
  /**
   * @brief Notifies this object that a referencing object is releasing
   * its reference of this object.
   */
  virtual unsigned long Release() = 0;
};
} // namespace atlast
} // namespace utils
/**
 * @def DEBUG_SMART_POINTERS
 * @brief If this symbol is defined, the AddRef and Release methods
 * will print some diagnostic messages.
 */
//#define DEBUG_SMART_POINTERS

/**
 * @class RefCountObjectImpl
 * @brief Template class that implements the atlast::utils::IRefCountObject
 * methods.
 *
 * The use of this class allows one to define abstract implementation
 * class that ultimately derive off of atlast::utils::IRefCountObject, yet never
 * implement those methods. When an instance of such a class is needed,
 * one can call the <code>Create</code> method on this class.
 *
 * \code
 * // Note: this is abstract!
 * class MyObject : public atlast::utils::IRefCountObject
 * {
 * public:
 *      void SomeMethod();
 * };
 * ...
 * MyObject* CreateMyObject()
 * {
 *      MyObject* pObj = RefCountObjectImpl<MyObject>::Create();
 *      return pObj;
 * }
 * \endcode
 * If the symbol <code>DEBUG_SMART_POINTERS</code>
 * is defined, the AddRef and Release methods
 * will print some diagnostic messages.
 *
 * @see atlast::utils::SmartPtr
 * @see atlast::utils::IRefCountObject
 * @see DEBUG_SMART_POINTERS
 */
template <typename Base>
class RefCountObjectImpl : public Base {
 public:
  RefCountObjectImpl()
    : m_nRefCount(0) {}

  virtual ~RefCountObjectImpl() {}
  /**
   * @brief Notifies this object that another object is referencing
   * this object.
   */
  virtual unsigned long AddRef() {
    m_nRefCount++;
#ifdef _WINDOWS
#   ifdef DEBUG_SMART_POINTERS
        USES_CONVERSION;
        CString thisName = A2T((char*)(typeid(*this).name()));
        TRACE(_T("AddRef: Reference count for %s is %d\n"), (LPCTSTR)thisName, m_nRefCount);
#   endif
#endif
    return m_nRefCount;
  }
  /**
   * @brief Notifies this object that a referencing object is releasing
   * its reference of this object.
   */
  virtual unsigned long Release() {
    ASSERT(m_nRefCount > 0);
    if (m_nRefCount == 0) {
      return 0L;
    }

    m_nRefCount--;
#ifdef _WINDOWS
#   ifdef DEBUG_SMART_POINTERS
        USES_CONVERSION;
        CString thisName = A2T((char*)(typeid(*this).name()));
        TRACE(_T("Release: Reference count for %s is %d\n"), (LPCTSTR)thisName, m_nRefCount);
#   endif
#endif
    // otherwise m_nRefCount might go out of 
    // scope with the delete and the return value is toasted.
    unsigned long refCount = m_nRefCount;
    if (m_nRefCount == 0) {
      delete this;
    }

    return refCount;
  }

  /**
   * @brief Creation method. 
   * 
   * This method will create an instance of the underlying object, but
   * wrapped by this template class, so that the <code>AddRef</code>
   * and <code>Release</code> methods are implemented.
   * @return A new instance of the <code>Base</code> object.
   */
  static RefCountObjectImpl<Base> *Create() {
    return new RefCountObjectImpl<Base>;
  }

 private:
  unsigned long m_nRefCount;
};

template <typename Base>
class RefCountObjectAggImpl : public Base {
protected:
  RefCountObjectAggImpl() {}
public:
  virtual ~RefCountObjectAggImpl() {}

  static RefCountObjectAggImpl<Base> *Create(atlast::utils::IRefCountObject
                                             *pParent) {
    ASSERT(pParent != NULL);

    RefCountObjectAggImpl<Base> *pObj = new RefCountObjectAggImpl<Base>;
    pObj->m_pParent = pParent;

    return pObj;
  }

  virtual unsigned long AddRef() {

#ifdef _WINDOWS
#   ifdef DEBUG_SMART_POINTERS
        TRACE(">> Aggregated Object: %s\n\t",typeid(*this).name());
#   endif
#endif

    unsigned long refCount = m_pParent->AddRef();

#ifdef _WINDOWS
#   ifdef DEBUG_SMART_POINTERS
        TRACE("\t...AddRef %s, count = %d\n", typeid(*this).name(), refCount);
#   endif
#endif

    return refCount;
  }
  virtual unsigned long Release() {

#ifdef _WINDOWS
#   ifdef DEBUG_SMART_POINTERS
        TRACE(">> Aggregated Object: %s\n\t",typeid(*this).name());
#   endif
#endif

    unsigned long refCount = m_pParent->Release();

#ifdef _WINDOWS
#   ifdef DEBUG_SMART_POINTERS
        TRACE("\t...Release %s, count = %d\n", typeid(*this).name(), refCount);
#   endif
#endif

    if (refCount == 0) {
      delete this;
    }
    return refCount;
  }
 private:
  atlast::utils::IRefCountObject *m_pParent;
};


#endif  // SKETCHUP_SOURCE_SKETCHUP_UTILS_IREFCOUNTOBJECT_H_
