// Copyright 2010 Google Inc. All Rights Reserved.

#ifndef SKETCHUP_SOURCE_SKETCHUP_UTILS_IVECTOROFSMARTPOINTERS_H_
#define SKETCHUP_SOURCE_SKETCHUP_UTILS_IVECTOROFSMARTPOINTERS_H_

#pragma once

#include "source/sketchup/utils/smartptr.h"
/**
 * \class IVectorOfSmartPointers
 * \brief Base class (interface) for a vector-like interface of smart pointers.
 * This object is itself a reference-count object.
 *
 * This interface is a STL vector-like object.  It defines
 * the methods:
 * <ul>
 *  <li> <code>void at(size_t indx)</code> - find the object at the
 *          given index in the vector.
 *  <li> <code>size_t size()</code> - returns the size of the vector.
 *  <li> <code>void push_back(<something>)</code> - appends the
 *          given object to the vector (the object must be of the type
 *          of the vector.
 *  <li> <code>void clear()</code> - clears the vector.
 * </ul>
 * 
 * This vector contains smart pointer objects of a particular type, as
 * determined by the template parameter.
 */
template <typename T, typename TPtr>
class IVectorOfSmartPointers : public atlast::utils::IRefCountObject {
 public:
  /**
   * \brief The standard destructor. An implementation would
   * call IRefCountObject::Release() on each of the held objects.
   */
  virtual ~IVectorOfSmartPointers() {}

  /**
   * \brief Returns the object at the given index.
   * \param indx The index into the array.  
   * \return The object at the indx'th entry of the array.
   * If < 0 or >= size,
   * the return value is NULL.
   */
  virtual TPtr at(size_t indx) const = 0;

  /**
   * \brief Returns the number of elements in the vector.
   * \return The size of the vector.
   */
  virtual size_t size() const = 0;

  /**
   * \brief Appends an object to the vector.
   * \param obj The object to append to the array.  The
   * validity of these objects (they may wrap NULL, e.g.) is
   * <b>not</b> verified.
   */
  virtual void push_back(T *obj) = 0;

  /**
   * \brief Used to clear the vector, releasing all the contained
   * objects.
   */
  virtual void clear() = 0;
};

#endif  // SKETCHUP_SOURCE_SKETCHUP_UTILS_IVECTOROFSMARTPOINTERS_H_
