// Copyright 2010 Google Inc. All Rights Reserved.

// Provide reference counted classes for storing data that can be large
// but which we want to be able to efficiently pass by value.
// This is mostly to provide for String and array data in CTypedValue.

#ifndef SKETCHUP_SOURCE_SKETCHUP_UTILS_REFCOUNTEDDATA_H_
#define SKETCHUP_SOURCE_SKETCHUP_UTILS_REFCOUNTEDDATA_H_

#include <vector>

#if defined(__LINUX__)
#include "source/sketchup/macwin/CString.h"
#endif

//=============================================================================
// Base class that does the reference counting
namespace atlast {
namespace utils {
class CRefCountBase {
 public:
  CRefCountBase() : m_nRef(1) {}
  virtual ~CRefCountBase() {}

  ULONG AddRef() {
    return ++m_nRef;
  }
  ULONG Release() {
    if (m_nRef > 0)
      m_nRef--;
    ULONG nref = m_nRef;
    if (0 == m_nRef)
      delete this;
    return nref;
  }
  ULONG NumRef() {
    return m_nRef;
  }

 protected:
  ULONG m_nRef;
};

//=============================================================================
// This is a helper class that holds the data.  It should never be used directly.

template<class T>
class CRefCountedArray;

template<class T>
class CRefCountedArrayData : public CRefCountBase {
 private:

  friend class CRefCountedArray<T>;
  friend class CTypedValue;

  typedef std::vector<T> vector_type;
  vector_type m_data;

  // Create an empty array
  CRefCountedArrayData() {}

  // Copy constructor
  CRefCountedArrayData(const CRefCountedArrayData &a) : m_data(a.m_data) {}

  // Create from a normal vector
  CRefCountedArrayData(const vector_type &a) : m_data(a) {}

  CRefCountedArrayData(size_t n) : m_data(n) {}

  CRefCountedArrayData(size_t n, T v) : m_data(n, v) {}

  ~CRefCountedArrayData() {}

};

//=============================================================================
// This is the actual reference counted array class that you should use.
template<class T>
class CRefCountedArray {
 public:
  typedef T value_type;
  typedef value_type *pointer;
  typedef const value_type *const_pointer;
  typedef value_type &reference;
  typedef const value_type &const_reference;
  typedef size_t size_type;
  typedef CRefCountedArrayData<value_type> data_type;

  // Create an empty array
  CRefCountedArray() : m_pData(NULL) {}

  // Copy constructor
  CRefCountedArray(const CRefCountedArray<T> &a) : m_pData(NULL) {
    SetData(a.m_pData);
  }

  CRefCountedArray(data_type *pData) : m_pData(NULL) {
    SetData(pData);
  }

  // Create an array with n elements
  CRefCountedArray(size_type n) {
    m_pData = new data_type(n);
  }
  CRefCountedArray(size_type n, value_type v) {
    m_pData = new data_type(n, v);
  }

  ~CRefCountedArray() {
    Release();
  }

  CRefCountedArray<T> &operator=(const CRefCountedArray<T> &a) {
    SetData(a.m_pData);
    return *this;
  }

  // Wrap some of the vector methods that we are interested in

  reference front() {
    CopyForWrite();
    return m_pData->m_data.front();
  }
  const_reference front() const {
    ASSERT(m_pData);
    return m_pData->m_data.front();
  }
  reference back() {
    CopyForWrite();
    return m_pData->m_data.back();
  }
  const_reference back() const {
    ASSERT(m_pData);
    return m_pData->m_data.back();
  }

  void push_back(T value) {
    CopyForWrite();
    m_pData->m_data.push_back(value);
  }

  void pop_back() {
    if (!m_pData->m_data.empty() ) {
      CopyForWrite();
      m_pData->m_data.pop_back();
    }
  }


  const_reference at(size_type nIndex) const {
    ASSERT(m_pData);
    return m_pData->m_data.at(nIndex);
  }
  reference operator[](size_type nIndex) {
    CopyForWrite();
    return m_pData->m_data[nIndex];
  }
  const_reference operator[](size_type nIndex) const {
    ASSERT(m_pData);
    return m_pData->m_data[nIndex];
  }

  size_type size() const {
    return m_pData ? m_pData->m_data.size() : 0;
  }

  size_type empty() const {
    return m_pData ? m_pData->m_data.empty() : true;
  }

  void clear() {
    CopyForWrite();
    m_pData->m_data.clear();
  }

 private:

  // Release the data
  void Release() {
    if (m_pData)
      m_pData->Release();
  }

  // Access the reference counted data
  void SetData(data_type *pData) {
    if (pData != m_pData) {
      Release();
      m_pData = pData;
      if (m_pData)
        m_pData->AddRef();
    }
  }

  // Copy the data before modifying it
  void CopyForWrite() {
    //printf("CopyForWrite: %x\n", this);
    if (NULL == m_pData) {
      m_pData = new data_type;
    } else if (m_pData->NumRef() > 1 ) {
      SetData(new data_type(*m_pData));
    }
  }

  data_type *m_pData;
};

//=============================================================================
// A reference counted string.  CString already uses reference counted data to
// store the actual string data.  It doesn't implement the AddRef and Release
// calls needed to make it work when you need a pointer to a CString.

class CRefCountedString : public CRefCountBase {
 public:
  CRefCountedString() {}
  CRefCountedString(const CString &s) : m_string(s) {}
  CRefCountedString(LPCTSTR s) : m_string(s) {}

  CString GetString() const {
    return m_string;
  }

 private:
  CString m_string;
};

}
} // namespace atlast::utils
//=============================================================================
#endif  // SKETCHUP_SOURCE_SKETCHUP_UTILS_REFCOUNTEDDATA_H_
