// Copyright 2010 Google Inc. All Rights Reserved.

#pragma once
#ifndef SKETCHUP_SOURCE_SKETCHUP_UTILS_SMARTPTR_H_
#define SKETCHUP_SOURCE_SKETCHUP_UTILS_SMARTPTR_H_

#include <typeinfo>
#include "source/sketchup/utils/irefcountobject.h"


/**
 * @class PointerCompare
 * @brief A comparitor function.
 * 
 * This comparison function checks to see whether the two
 * objects are equal using the native <code>operator==</code>
 * operator on the template class.
 */
template<class PTR>
class PointerCompare {
 public:
  static bool IsEqual(PTR t1, PTR t2) {
    return t1 == t2;
  }
  static bool LessThan(PTR t1, PTR t2) {
    return t1 < t2;
  }
};

namespace atlast {
namespace utils {
/**
 * @class SmartPtr
 * @brief A smart pointer that assumes the underlying object
 * has AddRef and Release methods.
 *
 * This smart pointer <b>will</b> call AddRef if constructed
 * with a pointer to the base class!  The assumption is, the 
 * construction of the base class object has not resulted in an AddRef
 * already being called.
 *
 * Example:
 * @code
 *  SmartPtr<ISketchUpApplication> pApp = new SkpApplication(...);
 *  pApp->DoSomething(); ...
 * @endcode
 * when pApp goes out of scope, the smart pointer will call Release()
 * on the SkpApplication object.
 *
 * @see IRefCountObject
 * @see PointerCompare
 */
template<typename PTR, typename Equality=PointerCompare<IRefCountObject*> >
class SmartPtr {
 public:
  /**
   * @brief Constructor from a pointer to the base object.
   * 
   * The underlying object's reference count is incremented by one.
   * @param pObj The object to wrap.
   */
  SmartPtr(PTR pObj = NULL)
    : m_pObj(pObj) {
    _AddRef();
  }

  /**
   * @brief Constructor given an arbitrary IRefCountObject object.
   *
   * The resulting object will wrap a non-NULL object iff the given
   * pointer can be converted to the template pointer type.
   *
   * @param pObj The object to wrap.
   */
  SmartPtr(IRefCountObject *pObj) {
    m_pObj = _QueryInterface(pObj);
    _AddRef();
  }
  /**
   * @brief Constructor taking an int - this is so that construction
   * with NULL works.
   *
   * This method does not check that the value of the int is non-zero.
   * No matter what value is passed in, the smart pointer wraps the
   * NULL pointer.
   * @param cNull Value-ignored.
   */
  SmartPtr(int cNull)
    : m_pObj(NULL) {}

  /**
   * @brief Constructor given another smart pointer. 
   * 
   * This method <i>will</i> call AddRef() on the underlying object.
   * @param ptr A smart pointer of the same type.
   */
  SmartPtr(const SmartPtr &ptr)
    : m_pObj(NULL) {
    m_pObj = ptr.m_pObj;
    _AddRef();
  }

  /**
   * @brief Constructor taking a smart pointer to another type.
   * 
   * If the other type is not compatible (cannot be cast to the
   * basic type of this smart pointer) then the resulting
   * smart pointer is NULL.
   * @param ptr A smart pointer of another type.
   */
  template<typename T2>
SmartPtr(const SmartPtr<T2> &ptr)
      : m_pObj(NULL) {
  m_pObj = _QueryInterface(ptr.m_pObj);
  _AddRef();
}


  /**
   * @brief Standard destructor.
   */
  virtual ~SmartPtr() {
    _Release();
  }

  /**
   * @brief Attaches this smart pointer to the given
   * object.
   *
   * If the given object is not compatible (cannot be cast to the
   * basic type of this smart pointer) then the resulting
   * smart pointer is NULL.
   * @param pObj A pointer to an IRefCountObject instance.
   * @param addRef If true (the default) the reference count
   * on the given object is incremented by one.
   */
  void Attach(IRefCountObject *pObj, bool addRef = true) {
    _Release();
    m_pObj = _QueryInterface(pObj);
    if (addRef) {
      _AddRef();
    }
  }

  /**
   * @brief Returns a pointer to the underlying object.
   * No AddRef() is called on the object.
   */
  operator PTR() const {
    return _QueryInterface(m_pObj);
  }

  /**
   * @brief Assigns the value of this smart pointer to 
   * another of the same type.
   *
   * The underlying object's reference count is incremented
   * to account for this smart pointer's referencing the
   * object.
   * @param ptr The other smart pointer.
   */
  SmartPtr &operator=(const SmartPtr &ptr) {
    if (m_pObj == ptr.m_pObj) {
      return *this; // nothing to do
    }
    _Release();
    m_pObj = ptr.m_pObj;
    _AddRef();
    return *this;
  }

  /**
   * @brief Assigns the value of this smart pointer to 
   * another smart pointer.
   *
   * The underlying object's reference count is incremented
   * to account for this smart pointer's referencing the
   * object.
   *
   * If the other type is not compatible (cannot be cast to the
   * basic type of this smart pointer) then the resulting
   * smart pointer is NULL.
   * @param ptr The other smart pointer.
   */
  template<typename T2>
SmartPtr<PTR> &operator=(const SmartPtr<T2> &ptr) {
  if (m_pObj == ptr.m_pObj) {
    return *this; // nothing to do
  }
  _Release();
  m_pObj = _QueryInterface(ptr.m_pObj);
  _AddRef();
  return *this;
}

  /**
   * @brief Assign the value of this smart pointer to a raw
   * pointer to an underlying object.
   *
   * The underlying object's reference count is incremented.
   * @param ptr The pointer to the object.
   */
  SmartPtr<PTR> &operator=(PTR pObj) {
    if (m_pObj == pObj) {
      return *this; // nothing to do
    }
    _Release();
    m_pObj = pObj;
    _AddRef();
    return *this;
  }

  /**
   * @brief Checks whether two smart pointers are equal.
   *
   * This method uses the <code>Equality</code> template
   * parameter.
   * @param ptr The other smart pointer.
   * @return true iff the two objects are equal by the
   * criterion set up by the <code>Equality</code> template
   * parameter.
   */
  bool operator==(const SmartPtr &ptr) {
    return Equality::IsEqual(m_pObj, ptr.m_pObj);
  }

  bool operator<(const SmartPtr &ptr) {
    return Equality::LessThan(m_pObj, ptr.m_pObj);
  }

  bool operator<=(const SmartPtr &ptr) {
    return Equality::LessThan(m_pObj, ptr.m_pObj) || Equality::IsEquals(m_pObj,
                                                                        ptr.m_pObj);
  }

  bool operator>(const SmartPtr &ptr) {
    return !Equality::LessThan(m_pObj,
                               ptr.m_pObj) && !Equality::IsEquals(m_pObj,
                                                                  ptr.m_pObj);
  }
  bool operator>=(const SmartPtr &ptr) {
    return !Equality::LessThan(m_pObj, ptr.m_pObj);
  }
  /**
   * @brief Checks whether two smart pointers are equal.
   *
   * This method uses the <code>Equality</code> template
   * parameter.
   * @param ptr The other smart pointer.
   * @return true iff the two objects are equal by the
   * criterion set up by the <code>Equality</code> template
   * parameter.
   */
  template<typename T2>
bool operator==(const SmartPtr<T2> &ptr) {
  return Equality::IsEqual(m_pObj, ptr.m_pObj);
}

  template<typename T2>
bool operator<(const SmartPtr<T2> &ptr) {
  return Equality::LessThan(m_pObj, ptr.m_pObj);
}

  /**
   * @brief Checks whether this smart pointers wraps
   * the given pointer.
   *
   * This method uses the <code>Equality</code> template
   * parameter.
   * @param ptr The other pointer.
   * @return true iff the two objects are equal by the
   * criterion set up by the <code>Equality</code> template
   * parameter.
   */
  bool operator==(PTR ptr) {
    return m_pObj == ptr;
  }

  /**
   * @brief Checks whether this smart pointer is wrapping
   * the NULL pointer.
   *
   * The actual value of the <code>cNull</code> parameter
   * is ignored, and assumed to be zero.
   * @param cNull Ignore value.
   * @return true iff the underlying object is NULL.
   */
  bool operator==(int cNull) {
    return m_pObj == NULL;
  }

  /**
   * @brief Checks whether two smart pointers are not equal.
   *
   * This method uses the <code>Equality</code> template
   * parameter.
   * @param ptr The other smart pointer.
   * @return true iff the two objects are not equal by the
   * criterion set up by the <code>Equality</code> template
   * parameter.
   */
  bool operator!=(const SmartPtr &ptr) {
    return m_pObj != ptr.m_pObj;
  }

  /**
   * @brief Checks whether this smart pointers wraps
   * the given pointer.
   *
   * This method uses the <code>Equality</code> template
   * parameter.
   * @param ptr The other pointer.
   * @return true iff the two objects are not equal by the
   * criterion set up by the <code>Equality</code> template
   * parameter.
   */
  bool operator!=(PTR ptr) {
    return m_pObj != ptr;
  }

  /**
   * @brief Checks whether this smart pointer is not wrapping
   * the NULL pointer.
   *
   * The actual value of the <code>cNull</code> parameter
   * is ignored, and assumed to be zero.
   * @param cNull Ignore value.
   * @return true iff the underlying object is not NULL.
   */
  bool operator!=(int cNull) {
    return m_pObj != NULL;
  }

  /**
   * @brief Allows access to the methods of the underlying
   * object.
   *
   * If the underlying object is NULL, this method will fail
   * in the operating specific manner associated with deferencing
   * a NULL pointer.
   * @return The pointer to the wrapped object.
   */
  PTR operator->() const {
    return _QueryInterface(m_pObj);
  }

  //    PTR* operator&()
  //    {
  //        return (PTR*)(&m_pObj);
  //    }
 protected:
  void _AddRef() {
    if (m_pObj) {
      m_pObj->AddRef();
    }
  }

  void _Release() {
    if (m_pObj) {
      m_pObj->Release();
    }
  }

  PTR _QueryInterface(IRefCountObject *pObj) const {
    return dynamic_cast<PTR>(pObj);
  }
 public:
  /**
   * @brief Raw access to the underlying object as an IRefCountObject.
   *
   * This parameter should be considered private.
   */
  IRefCountObject *m_pObj;
};
}
} // namespace atlast::utils

#endif  // SKETCHUP_SOURCE_SKETCHUP_UTILS_SMARTPTR_H_
