// Copyright 2010 Google Inc. All Rights Reserved.

#ifndef SKETCHUP_SOURCE_SKETCHUP_UTILS_SMARTVECTOR_H_
#define SKETCHUP_SOURCE_SKETCHUP_UTILS_SMARTVECTOR_H_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#include <vector>
/**
* \class SmartVector
* \brief A templated implementation of the IVectorOfSmartPointers class.
* 
* Usage: 
* \code
typedef SkpVertexVector SmartVector<ISketchUpVertex, ISketchUpVertexPtr, ISketchUpVertexVector>;
ISketchUpVertexVectorPtr pVector = SkpVertexVector::Create();
* \endcode
*/
template <typename WrappedType, typename WrappedTypePtr, typename VectorType>
class SmartVector :
    public VectorType {
 private:
  std::vector<WrappedType*> m_objs;
 protected:
  SmartVector() {}
 public:
  /**
   * \brief Create a vertex vector
   * \return A new SkpVertexVector with Reference counting implemented.
   */
  static SmartVector<WrappedType, WrappedTypePtr, VectorType> *Create() {
    return RefCountObjectImpl<SmartVector<WrappedType, WrappedTypePtr,
      VectorType> >::Create();
  }

  /**
   * \brief The standard destructor. An implementation would
   * call IRefCountObject::Release() on each of the held objects.
   */
  virtual ~SmartVector() {
    clear();
  }

  /**
   * \brief Returns the object at the given index.
   * \param indx The index into the array.  
   * \return The object at the indx'th entry of the array.
   * If < 0 or >= size,
   * the return value is NULL.
   */
  virtual WrappedTypePtr at(size_t indx) const {
    if (indx >= 0 && indx < size()) {
      // Since we are returning a smart pointer, we
      // get the AddRef called at the time that
      // smart pointer is constructed
      return m_objs[indx];
    }
    return NULL;
  }
  /**
   * \brief Returns the number of elements in the vector.
   * \return The size of the vector.
   */
  virtual size_t size() const {
    return m_objs.size();
  }

  /**
   * \brief Appends an object to the vector.
   * \param obj The object to append to the array.  The
   * validity of these objects (they may wrap NULL, e.g.) is
   * <b>not</b> verified.
   */
  virtual void push_back(WrappedType *pObj) {
    if (pObj != NULL) {
      m_objs.push_back(pObj);
      pObj->AddRef();
    }
  }
  /**
   * \brief Used to clear the vector, releasing all the contained
   * objects.
   */
  virtual void clear() {
    for (unsigned int i = 0; i < m_objs.size(); i++) {
      if (m_objs[i] != NULL) {
        m_objs[i]->Release();
      }
    }
    m_objs.clear();
  }
};

#endif  // SKETCHUP_SOURCE_SKETCHUP_UTILS_SMARTVECTOR_H_
