// Copyright 2010 Google Inc. All Rights Reserved.

#ifndef SKETCHUP_SOURCE_SKETCHUP_UTILS_TYPEDVALUE_H_
#define SKETCHUP_SOURCE_SKETCHUP_UTILS_TYPEDVALUE_H_

#ifndef __LINUX__
#pragma warning( disable : 4290 )
#endif 

#include "source/common/utils/color.h"
#include "source/common/geometry/point3d.h"
#include "source/common/geometry/vector3d.h"
#include "source/common/geometry/unitvector3d.h"
#include "source/common/geometry/transformation.h"
#include "source/sketchup/utils/refcounteddata.h"
#include <time.h>

#if defined(__APPLE__) || defined(__LINUX__)
#include "source/sketchup/macwin/Variant.h"
#endif

class CTime;


//===================================================================
namespace atlast {
namespace utils {

enum EValueType {
  kVtEmpty = 0,   // value not set
  kVtNull = 1,
  kVtI1 = 2,      // BYTE
  kVtI2 = 3,      // short
  kVtI4 = 4,      // long
  kVtR4 = 5,      // float
  kVtR8 = 6,      // double
  kVtBool = 7,    // bool
  kVtColor = 8,   // COLORREF
  kVtTime = 9,    // time and date
  kVtString = 10, // string
  kVtArray = 11,  // Array of CTypedValue
  // The following types are all variations of kVtR8 for different units
  // The data is always in the R8 value of the union.
  kVtLength = 12, // Length units = inches
  kVtArea = 13,   // area units
  kVtWeight = 14, // weight units
  kVtForce = 15,  // force units
  kVtCurrency = 16, // This is usual some kind of fixed point decimal
  kVtPoint = 17,    // actually stored as an array
  kVtVector = 18,    // actually stored as an array
  kVtUnitVector = 19,    // actually stored as an array
  kVtTransform = 20   // actually stored as an array
};

// TODO: Add support for points and vectors.

class CTypedValue;
typedef CRefCountedArray<CTypedValue> CTypedArray;
typedef CRefCountedArrayData<CTypedValue> CTypedArrayData;
typedef std::vector<CTypedValue> CTypedValueVector;
typedef CRefCountedArrayData<double> CCoordData;

//-------------------------------------------------------------------

// Class used to throw an exception if you cannot convert a CTypedValue
// to a desired type.

class CConversionError {
 public:
  CConversionError(EValueType valueType) : m_type(valueType) {};
  CConversionError() : m_type(kVtEmpty) {};

  ~CConversionError() {}

  EValueType m_type;
};

//===================================================================

class CTypedValue {
 public:

  CTypedValue() : m_type(kVtEmpty) {
    m_value.I4 = 0;
  }

  CTypedValue(BYTE value) : m_type(kVtI1) {
    m_value.I1 = value;
  }

  CTypedValue(short value) : m_type(kVtI2) {
    m_value.I2 = value;
  }

  CTypedValue(long value) : m_type(kVtI4) {
    m_value.I4 = value;
  }

  CTypedValue(float value) : m_type(kVtR4) {
    m_value.R4 = value;
  }

  CTypedValue(double value) : m_type(kVtR8) {
    m_value.R8 = value;
  }

  CTypedValue(bool value) : m_type(kVtBool) {
    m_value.boolVal = value;
  }

  CTypedValue(const CTime &value);
  CTypedValue(const CColor &value);
  CTypedValue(LPCWSTR value);
  CTypedValue(LPCSTR value);
  CTypedValue(const CString &value);
  CTypedValue(const CTypedValue &value);
  CTypedValue(const CTypedValueVector &value);
  CTypedValue(const atlast::geometry::CPoint3d &value);
  CTypedValue(const atlast::geometry::CVector3d &value);
  CTypedValue(const atlast::geometry::CUnitVector3d &value);
  CTypedValue(const atlast::geometry::CTransformation &value);

  ~CTypedValue();

  void Set(bool value) {
    ReleaseData();
    m_type = kVtBool;
    m_value.boolVal = value;
  }
  void Set(short value) {
    ReleaseData();
    m_type = kVtI2;
    m_value.I2 = value;
  }
  void Set(int value) {
    ReleaseData();
    m_type = kVtI4;
    m_value.I4 = value;
  }
  void Set(long value) {
    ReleaseData();
    m_type = kVtI4;
    m_value.I4 = value;
  }
  void Set(float value) {
    ReleaseData();
    m_type = kVtR4;
    m_value.R4 = value;
  }
  void Set(double value) {
    ReleaseData();
    m_type = kVtR8;
    m_value.R8 = value;
  }
  void Set(CColor value) {
    ReleaseData();
    m_type = kVtColor;
    m_value.color = value.GetRGBA();
  }
  void Set(CString value);
  void SetTime(time_t value) {
    ReleaseData();
    m_type = kVtTime;
    m_value.time = value;
  }
  void Set(const CTime &value);
  void Set(const atlast::geometry::CPoint3d &value);
  void Set(const atlast::geometry::CVector3d &value);
  void Set(const atlast::geometry::CUnitVector3d &value);
  void Set(const atlast::geometry::CTransformation &value);
  void SetLength(double value) {
    ReleaseData();
    m_type = kVtLength;
    m_value.R8 = value;
  }

  bool Get(bool &value) const;
  bool Get(short &value) const;
  bool Get(int &value) const;
  bool Get(long &value) const;
  bool Get(float &value) const;
  bool Get(double &value) const;
  bool Get(CColor &value) const;
  bool Get(CTime &value) const;
  bool Get(CString &value) const;
  bool Get(atlast::geometry::CPoint3d &value) const;
  bool Get(atlast::geometry::CVector3d &value) const;
  bool Get(atlast::geometry::CUnitVector3d &value) const;
  bool Get(atlast::geometry::CTransformation &value) const;

  bool Set(VARIANT var);
  bool Get(VARIANT *pv) const;

  CTypedValue &operator=(BYTE value) {
    ReleaseData();
    m_type = kVtI1;
    m_value.I1 = value;
    return *this;
  }

  CTypedValue &operator=(short value) {
    Set(value);
    return *this;
  }

  CTypedValue &operator=(long value) {
    Set(value);
    return *this;
  }

  CTypedValue &operator=(int value) {
    Set(value);
    return *this;
  }

  CTypedValue &operator=(float value) {
    ReleaseData();
    m_type = kVtR4;
    m_value.R4 = value;
    return *this;
  }

  CTypedValue &operator=(double value) {
    Set(value);
    return *this;
  }

  CTypedValue &operator=(bool value) {
    Set(value);
    return *this;
  }

  CTypedValue &operator=(const atlast::geometry::CPoint3d &value) {
    Set(value);
    return *this;
  }

  CTypedValue &operator=(const atlast::geometry::CVector3d &value) {
    Set(value);
    return *this;
  }

  CTypedValue &operator=(const atlast::geometry::CUnitVector3d &value) {
    Set(value);
    return *this;
  }

  CTypedValue &operator=(const atlast::geometry::CTransformation &value) {
    Set(value);
    return *this;
  }

  CTypedValue &operator=(const CColor &value);
  CTypedValue &operator=(LPCWSTR value);
  CTypedValue &operator=(LPCSTR value);
  CTypedValue &operator=(const CTypedValue &value);
  CTypedValue &operator=(const CTypedValueVector &value);

  EValueType GetType() const {
    return m_type;
  }

  operator BYTE() const
  throw(CConversionError);
  operator short() const
  throw(CConversionError);
  operator long() const
  throw(CConversionError);
  operator float() const
  throw(CConversionError);
  operator double() const
  throw(CConversionError);
  operator bool() const
  throw(CConversionError);

  // Using a cast doesn't work for these on the Mac
  bool GetColor(CColor &color) const;
  CColor GetColor() const
  throw(CConversionError);

  bool GetString(CString &s) const;
  CString GetString() const
  throw(CConversionError);

  CTypedArray GetArray() const
  throw(CConversionError);

  // Support transformation of typed values.
  // Since this class isn't undoable, we cannot backup the value
  // before doing the transform, so we need to back up the object that
  // uses the CTypedValue.  Most kinds of CTypedValue are not transformable
  // though so I only want to do hte backup when it is needed.  The first
  // method will tell you if the Transform method will actually do anything.
  bool CanBeTransformed() const;
  bool Transform(const atlast::geometry::CTransformation &tran);

#ifdef __AFX_H__
	friend CArchive& operator<<(CArchive& ar, const CTypedValue& value);
	friend CArchive& operator>>(CArchive& ar, CTypedValue& value);

    // Implement as member functions so that we can access the private data
    CArchive& SaveTo(CArchive& ar) const;
    CArchive& ReadFrom(CArchive& ar);
#endif //__AFX_H__

 protected: // methods

  void ReleaseData();

 public: // data

  EValueType m_type;
  union {
    BYTE I1;
    short I2;
    long I4;
    float R4;
    double R8;
    bool boolVal;
    COLORREF color;
    time_t time;
    CRefCountedString *pstring;
    CTypedArrayData *parray;
    CCoordData *pcoords;
  } m_value;
};
}
} // namespace atlast::utils
//===================================================================
#endif  // SKETCHUP_SOURCE_SKETCHUP_UTILS_TYPEDVALUE_H_
